package mice.cdb.service;

import java.sql.Timestamp;

/**
 * Methods used to validate user input.
 * 
 */
public class Validator {

    /**
     * Check String only contains valid charters. Alpha numeric, white space, _,
     * +, -, :, . or /
     * 
     * @param s
     *            the String to validate
     * @throws CDBException
     *             String contains a non valid character
     */
    public void validateString(String s) throws CDBException {
        // TODO compile pattern
        // String p = "^[a-zA-Z0-9_]*$";
        // Pattern pattern = Pattern.compile(p, Pattern.CASE_INSENSITIVE);
        // s.matches(pattern);
        if (s == null || s.isEmpty()) {
            return;
        }
        if (s.matches("^[\\s\\w:;,~=\\-\\+\\\\.\\\\/%]*$")) {
            return;
        } else {
            throw new CDBException("String contained non-aphanumeric/non-white space characters: " + s);
        }
    }

    /**
     * Check String only contains valid charters. Alpha numeric, white space, _,
     * +, -, :, . or / and it is not empty.
     * 
     * @param s
     *            the String to validate
     * @throws CDBException
     *             String contains a non valid character
     */
    public void validateStringNonBlank(String s) throws CDBException {
        if (s == null || s.isEmpty()) {
            throw new CDBException("String cannot be null or empty");
        }
        validateString(s);
    }

    /**
     * Check String is not empty or <code>null</code>.
     * 
     * @param s
     *            the String to validate
     * @throws CDBException
     *             String is empty or <code>null</code>
     */
    public void validateNonBlank(String s) throws CDBException {
        if (s == null | s.isEmpty()) {
            throw new CDBException("String cannot be null or empty");
        }
    }

    /**
     * Check byte array is not empty or <code>null</code>.
     * 
     * @param b
     *            the byte array to validate
     * @throws CDBException
     *             byte array is empty or <code>null</code>
     */
    public void validateNonBlank(byte[] b) throws CDBException {
        if (b == null | b.length == 0) {
            throw new CDBException("Byte array cannot be null or empty");
        }
    }

    /**
     * Returns the time stamp value.
     * 
     * @param timestamp
     *            a String containing a timestamp
     * @return a Timestamp object
     * @throws CDBException
     *             String is not a valid timestamp.
     */
    public Timestamp getTimeStamp(String timestamp) throws CDBException {
        try {
            return Timestamp.valueOf(timestamp.trim());
        } catch (NumberFormatException e) {
            // occurs when there is a trailing character
            // i.e. "2010-10-25 12:03:03.18700000Z"
            throw new CDBException("Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]");
        } catch (NullPointerException e) {
            throw new CDBException("Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]");
        } catch (IllegalArgumentException e) {
            throw new CDBException(e.getMessage());
        }
    }

    /**
     * Returns the integer value.
     * 
     * @param s
     *            a String representation of an integer
     * @return an integer
     * @throws CDBException
     */
    public int getInt(String s) throws CDBException {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            throw new CDBException("Value:" + s + " is not a valid int");
        }
    }

    /**
     * Returns the double value.
     * 
     * @param s
     *            a String representation of an integer
     * @return an integer
     * @throws CDBException
     */
    public double getDouble(String s) throws CDBException {
        try {
            return Double.parseDouble(s);
        } catch (NumberFormatException e) {
            throw new CDBException("Value:" + s + " is not a valid double");
        }
    }

}
