package mice.cdb.service.alarmhandler;

import java.sql.Timestamp;
import java.util.Vector;

/**
 * Container for <code>Alarms</code>.
 */
@SuppressWarnings("serial")
public class AlarmList extends Vector<Alarm> {

    private Timestamp m_creationTime;
    private String m_tag;

    /**
     * Constructs an AlarmList object.
     */
    public AlarmList() {
    }

    /**
     * Constructs an AlarmList object.
     * 
     * @param creationTime
     *            a timestamp containing the creation time of the AlarmList
     * @param tag
     *            the name of the AlarmList
     */
    protected AlarmList(Timestamp creationTime, String tag) {
        m_creationTime = creationTime;
        m_tag = tag;
    }

    /**
     * Get the INSERT statements for the AlarmList
     * 
     * @param creationTime
     *            the time this version of the tag was created
     * @param tag
     *            the name of the tag
     * @return a string containing a series of INSERT statements
     */
    protected String getInsertQuery(Timestamp creationTime, String tag) {
        StringBuilder insert = new StringBuilder();

        for (int i = 0; i < size(); i++) {
            insert.append(get(i).getInsertQuery(creationTime, tag));
        }

        return insert.toString();
    }

    /**
     * Get the the XML representation of an AlarmList.
     * 
     * @return a string containing the XML representation of an AlarmList
     */
    protected String toXMLString() {
        StringBuilder xml = new StringBuilder();
        xml.append("<alarmhandler creationtime='").append(m_creationTime.toString()).append("' tag='").append(m_tag)
                .append("'>");

        for (int i = 0; i < size(); i++) {
            xml.append(get(i).toXMLString());
        }

        xml.append("</alarmhandler>");
        return xml.toString();
    }

}
