package mice.cdb.service.calibration;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>CalibrationSuperMouse</code> interface is used to store and
 * retrieve calibration data. For the trackers data is described by its location
 * with reference to bank, board and channel. Calibration data is set/updated
 * with {@link #setTracker(String, String) setTracker}. Old versions of the data
 * are stored for diagnostic purposes. Data can be retrieved for a given time
 * using {@link #getCalibrationForDate(String, String) getCalibrationForDate}
 * and for a given run using {@link #getCalibrationForRun(String, String)
 * getCalibrationForRun}
 * </p>
 * 
 * <p>
 * The returned XML may contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface CalibrationSuperMouse extends Calibration {

    /**
     * Update the data for the given board, bank, channel. A timestamp is
     * associated with the data.
     * 
     * @param device
     *            the name of the tracker
     * @param xml
     *            a string containing XML representing the channel
     *            <dl>
     *            <dd><code>&lt;calibrations&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tracker board='n' bank='n' channel='n' adcPedestal='n.n' adcGain='n.n' tdcPedestal='n.n' tdcSlope='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tracker board='n' bank='n' channel='n' adcPedestal='n.n' adcGain='n.n' tdcPedestal='n.n' tdcSlope='n.n'/&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/calibrations&gt;</code></dd>
     *            </dl>
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setTracker(String device, String xml);

    /**
     * Update the data for the given device. A timestamp is associated with the
     * data. Using a byte array enables the user to pass in the data in a zipped
     * format if so required.
     * 
     * @param device
     *            the name of the detector
     * @param type
     *            the type of the calibration
     * @param validFromTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC.
     *            This should represent the time from which this data set is
     *            considered to be valid.
     * 
     * @param data
     *            a byte array containing a calibration data set
     * 
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setDetector(String device, String type, String validFromTime, byte[] data);

}
