package mice.cdb.service.control;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>ControlSuperMouse</code> interface is used to store and retrieve
 * control parameter values. An individual parameter can be added using
 * {@link #setParameter(String, String, String, String, String) setParameter} or
 * updated using
 * {@link #updateParameter(String, String, String, String, String)
 * updateParameter}. Old control parameter values are stored for diagnostic
 * purposes. A set of control parameter values can be retrieved for a given time
 * using {@link #getPreviousSettings(String) getPreviousSettings}
 * </p>
 * 
 * <p>
 * The returned XML may contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface ControlSuperMouse extends Control {

    /**
     * Set the parameter value of an individual parameter for the given channel.
     * A timestamp is associated with the parameter. <b>N.B.</b> There must NOT
     * already be a value set for the parameter otherwise an error will be
     * reported.
     * 
     * @param crate
     *            a string containing the number of the crate
     * @param module
     *            a string containing the number of the module
     * @param channel
     *            a string containing the number of the channel
     * @param parameterName
     *            a string containing the name of the parameter
     * @param parameterValue
     *            a string containing the value to set
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setParameter(String crate, String module, String channel, String parameterName,
            String parameterValue);

    /**
     * Update the parameter value of an individual parameter for the given
     * channel. The timestamp associated with the parameter is updated to the
     * current time.<b>N.B.</b> There must already be a value set for the
     * parameter otherwise an error will be reported.
     * 
     * @param crate
     *            a string containing the number of the crate
     * @param module
     *            a string containing the number of the module
     * @param channel
     *            a string containing the number of the channel
     * @param parameterName
     *            a string containing the name of the parameter
     * @param parameterValue
     *            a string containing the value to set
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String updateParameter(String crate, String module, String channel, String parameterName,
            String parameterValue);

}
