package mice.cdb.service.coolingchannel;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>Coolingchannel</code> interface is used to retrieve data about
 * cooling channel settings.
 * </p>
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 * 
 */
@WebService
public interface Coolingchannel {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the cooling channel data for all runs.
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;coolingchannels&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;coolingchannel validfromtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' validuntiltime='yyyy-mm-dd hh:mm:ss[.fffffffff]'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnets&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnets&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/coolingchannel&gt;</code></dd>
     *         <dd><code>&lt;/coolingchannels&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getAllCoolingchannels();

    /**
     * Get the cooling channel data for the given run.
     * 
     * @param runNo
     *            the run number
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;coolingchannels&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;coolingchannel validfromtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' validuntiltime='yyyy-mm-dd hh:mm:ss[.fffffffff]'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnets&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnets&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/coolingchannel&gt;</code></dd>
     *         <dd><code>&lt;/coolingchannels&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getCoolingchannelForRun(String runNo);

    /**
     * Get the cooling channel data for the given time period.
     * 
     * @param timestamp
     *            : a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in
     *            UTC
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;coolingchannels&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;coolingchannel validfromtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' validuntiltime='yyyy-mm-dd hh:mm:ss[.fffffffff]'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnets&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnets&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/coolingchannel&gt;</code></dd>
     *         <dd><code>&lt;/coolingchannels&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getCoolingchannelForDate(String timestamp);

    /**
     * Get the cooling channel data for the given tag.
     * 
     * @param tag
     *            : tag name
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;coolingchannels&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;coolingchannel validfromtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' validuntiltime='yyyy-mm-dd hh:mm:ss[.fffffffff]'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnets&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' calibration='n.n' ilim='n.n' iset='1.3' rate='n.n' stability='n.n' vlim='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnets&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/coolingchannel&gt;</code></dd>
     *         <dd><code>&lt;/coolingchannels&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getCoolingchannelForTag(String tag);

    /**
     * List the available tags
     * 
     * @return a string containing XML representing the cooling channel with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;tagNames&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tag name='xxx'/&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tag name='xxx'/&gt;</code></dd>
     *         <dd><code>&lt;/tagNames&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String listTags();

}
