package mice.cdb.service.coolingchannel;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>CoolingchannelSuperMouse</code> interface is used to store and
 * retrieve data about cooling channel settings.
 * </p>
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * The returned XML will contain a message enclosed in one of three tags: ok,
 * warning or error. A tag of warning indicates that the problem maybe transient
 * and retrying the request MAY succeed. A tag of error maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 * 
 */
@WebService
public interface CoolingchannelSuperMouse extends Coolingchannel {

    /**
     * Add the parameters associated with the cooling channel.
     * 
     * @param xml
     *            a string containing XML representing the cooling channel with
     *            the format of the XML as
     *            <dl>
     *            <dd><code>&lt;coolingchannel&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnet&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnet&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/coolingchannel&gt;</code></dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setCoolingchannel(String xml);

    /**
     * Add the parameters associated with a cooling channel tag.
     * 
     * @param xml
     *            a string containing XML representing the cooling channel with
     *            the format of the XML as
     *            <dl>
     *            <dd><code>&lt;tag name='xxx'&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnet&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' mode='xxx' polarity='xxx'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;coil name='xxx' iLim='n.n' vLim='n.n' rate='n.n' iSet='n.n' stability='n.n' calibration='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/magnet&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/tag&gt;</code></dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setCoolingchannelTag(String xml);

}
