package mice.cdb.service.geometry;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>GeometrySuperMouse</code> interface is used to store and retrieve
 * GDML data sets. A GDML data set can be stored using
 * {@link #setGDML(byte[], String, String, String) setGDML}, where the user
 * defines the time the data was valid from and provides a brief description of
 * the change. The data set will be assigned an <code>id</code> and
 * <code>creation time</code>. Use {@link Geometry#getIds(String, String)
 * getIds} to retrieve a list of ids.
 * </p>
 * 
 * <p>
 * Setting another GDML data set with a later <code>valid from time</code>
 * implies that the earlier data set is no longer valid after the later
 * <code>valid from time</code>. It is possible to "update" an existing GDML
 * data set by setting a new one with the same <code>valid from time</code> as
 * an existing one. The old data set is kept and the GDML data sets can be
 * distinguished by their <code>creation times</code>.
 * </p>
 * 
 * <p>
 * The returned XML may contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface GeometrySuperMouse extends Geometry {

    /**
     * Set the GDML data set. Using a byte array enables the user to pass in the
     * data in a zipped format if so required.
     * 
     * @param gdml
     *            a byte array containing a GDML data set
     * @param validFromTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC.
     *            This should represent the time from which this data set is
     *            considered to be valid.
     * @param notes
     *            a string containing a description of the change
     * @param technicalDrawingName
     *            a string containing the name of the technical drawing
     * 
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     * 
     * @see #getCurrentGDML()
     * @see #getGDMLForId(java.lang.String)
     */
    @WebMethod
    public abstract String setGDML(byte[] gdml, String validFromTime, String notes, String technicalDrawingName);

}
