package mice.cdb.service.geometry;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;

/**
 * An implementation of the <code>GeometrySuperMouse</code>.
 * 
 * @see mice.cdb.service.geometry.GeometrySuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.geometry.GeometrySuperMouse")
public class GeometrySuperMouseImpl extends GeometryImpl implements GeometrySuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Construct a Geometry with super mouse powers.
     */
    public GeometrySuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String setGDML(byte[] gdml, String validFromTime, String notes, String technicalDrawingName) {
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        int inserted = 0;
        Connection connection = null;

        try {
            m_validator.validateNonBlank(gdml);
            Timestamp validFrom = m_validator.getTimeStamp(validFromTime);
            m_validator.validateString(notes);
            m_validator.validateString(technicalDrawingName);
            connection = m_mdb.getConnect();
            String insert = "INSERT INTO " + GeometryConstants.TABLE_GEOMETRY + " (" + GeometryConstants.COLUMN_GDML
                    + ", " + GeometryConstants.COLUMN_VALIDFROMTIME + ", " + GeometryConstants.COLUMN_CREATIONTIME
                    + ", " + GeometryConstants.COLUMN_NOTES + ", " + GeometryConstants.COLUMN_TECHNICAL_DRAWING_NAME
                    + ") VALUES (?, '" + validFrom + "', '" + creationTime + "', '" + notes + "', '"
                    + technicalDrawingName + "');";
            m_log.debug("insert:" + insert);
            PreparedStatement ps = m_mdb.prepareStatement(connection, insert);
            ps.setBytes(1, gdml);
            inserted = ps.executeUpdate();
            ps.close();
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } catch (SQLException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        if (inserted == 1) {
            m_log.info("Inserted GDML, " + gdml.length + " bytes");
            return "<ok>Inserted GDML, " + gdml.length + " bytes</ok>";
        } else {
            m_log.info("GDML not inserted");
            return "<error>GDML not inserted</error>";
        }
    }
}
