package mice.cdb.service.parser;

import mice.cdb.service.alarmhandler.Alarm;
import mice.cdb.service.alarmhandler.AlarmHandlerConstants;
import mice.cdb.service.alarmhandler.AlarmList;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class AlarmHandlerParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private AlarmList m_alarmList;
    private Alarm m_alarm;
    private StringBuilder m_elemantValue;

    /**
     * Default constructor
     */
    public AlarmHandlerParser() {
    }

    /**
     * Get the AlarmList containing the parsed XML.
     * 
     * @return the parsed XML as an AlarmList
     */
    public AlarmList getAlarmList() {
        return m_alarmList;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(AlarmHandlerConstants.ALARMHANDLER)) {
            createAlarmList(attr);
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.ALARM)) {
            createAlarm(attr);
        } else if (m_alarm == null) {
            throw new SAXException("The element '" + qName + "' appeared before the element '"
                    + AlarmHandlerConstants.ALARM + "'");
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.NAME)) {
            m_elemantValue = new StringBuilder();
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.HIHI)) {
            m_elemantValue = new StringBuilder();
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.HI)) {
            m_elemantValue = new StringBuilder();
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.LO)) {
            m_elemantValue = new StringBuilder();
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.LOLO)) {
            m_elemantValue = new StringBuilder();
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        m_log.debug("End of " + qName);
        if (qName.equalsIgnoreCase(AlarmHandlerConstants.ALARM)) {
            addAlarm();
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.NAME)) {
            validateString(m_elemantValue.toString());
            m_alarm.setName(m_elemantValue.toString());
            m_log.debug("name: " + m_elemantValue);
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.HIHI)) {
            m_alarm.setHiHi(parseDouble(m_elemantValue.toString()));
            m_log.debug("hihi: " + m_elemantValue);
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.HI)) {
            m_alarm.setHi(parseDouble(m_elemantValue.toString()));
            m_log.debug("hi: " + m_elemantValue);
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.LO)) {
            m_alarm.setLo(parseDouble(m_elemantValue.toString()));
            m_log.debug("lo: " + m_elemantValue);
        } else if (qName.equalsIgnoreCase(AlarmHandlerConstants.LOLO)) {
            m_alarm.setLoLo(parseDouble(m_elemantValue.toString()));
            m_log.debug("lolo: " + m_elemantValue);
        }
    }

    @Override
    public void characters(char ch[], int start, int length) {
        m_elemantValue = new StringBuilder();
        if (m_elemantValue != null) {
            m_elemantValue.append(new String(ch, start, length));
        }
    }

    private void createAlarmList(Attributes attr) {
        m_alarmList = new AlarmList();
        m_log.debug("created new AlarmList");
    }

    private void createAlarm(Attributes attr) throws SAXException {
        if (m_alarmList == null) {
            throw new SAXException("The element '" + AlarmHandlerConstants.ALARM + "' appeared before the element '"
                    + AlarmHandlerConstants.ALARMHANDLER + "'");
        }
        m_alarm = new Alarm();
        m_log.debug("created new Alarm");
    }

    private void addAlarm() throws SAXException {
        if (!m_alarm.validate()) {
            throw new SAXException("Error with values of alarm " + m_alarm.getName());
        }
        m_alarmList.add(m_alarm);
        m_alarm = null;
        m_log.debug("added new Alarm");
    }
}
