package mice.cdb.service.parser;

import java.util.LinkedList;
import java.util.List;

import mice.cdb.service.cabling.CablingConstants;
import mice.cdb.service.cabling.TrackerData;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class CablingParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private List<TrackerData> m_trackerData = new LinkedList<TrackerData>();

    /**
     * Default constructor
     */
    public CablingParser() {
    }

    /**
     * Get the parsed XML.
     * 
     * @return the parsed XML as a TrackerData
     */
    public List<TrackerData> getTrackerData() {
        return m_trackerData;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(CablingConstants.CABLING)) {
            m_trackerData = new LinkedList<TrackerData>();
        } else if (qName.equalsIgnoreCase(CablingConstants.TRACKER)) {
            addTracker(attr);
        }
    }

    private void addTracker(Attributes attr) throws SAXException {
        TrackerData trackerData = new TrackerData();
        trackerData.setVlsbComputerId(parseInt(attr.getValue(CablingConstants.VLSB_COMPUTER_ID)));
        trackerData.setVlsbGeoNumber(parseInt(attr.getValue(CablingConstants.VLSB_GEO_NUMBER)));
        trackerData.setVlsbChannel(parseInt(attr.getValue(CablingConstants.VLSB_CHANNEL)));
        trackerData.setTrackerNo(parseInt(attr.getValue(CablingConstants.TRACK_NO)));
        trackerData.setStation(parseInt(attr.getValue(CablingConstants.STATION)));
        trackerData.setPlane(parseInt(attr.getValue(CablingConstants.PLANE)));
        trackerData.setChannel(parseInt(attr.getValue(CablingConstants.CHANNEL)));
        m_trackerData.add(trackerData);
    }

}
