package mice.cdb.service.parser;

import java.util.LinkedList;
import java.util.List;

import mice.cdb.service.calibration.CalibrationConstants;
import mice.cdb.service.calibration.TrackerData;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class CalibrationParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private List<TrackerData> m_trackerData = new LinkedList<TrackerData>();

    /**
     * Default constructor
     */
    public CalibrationParser() {
    }

    /**
     * Get the parsed XML.
     * 
     * @return the parsed XML as a TrackerData
     */
    public List<TrackerData> getTrackerData() {
        return m_trackerData;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(CalibrationConstants.CALIBRATIONS)) {
            m_trackerData = new LinkedList<TrackerData>();
        } else if (qName.equalsIgnoreCase(CalibrationConstants.TRACKER)) {
            addTracker(attr);
        }
    }

    private void addTracker(Attributes attr) throws SAXException {
        TrackerData trackerData = new TrackerData();
        trackerData.setBank(parseInt(attr.getValue(CalibrationConstants.BANK)));
        trackerData.setBoard(parseInt(attr.getValue(CalibrationConstants.BOARD)));
        trackerData.setChannel(parseInt(attr.getValue(CalibrationConstants.CHANNEL)));
        trackerData.setAdcPedestal(parseDouble(attr.getValue(CalibrationConstants.ADC_PEDESTAL)));
        trackerData.setAdcGain(parseDouble(attr.getValue(CalibrationConstants.ADC_GAIN)));
        trackerData.setTdcPedestal(parseDouble(attr.getValue(CalibrationConstants.TDC_PEDESTAL)));
        trackerData.setTdcSlope(parseDouble(attr.getValue(CalibrationConstants.TDC_SLOPE)));
        m_trackerData.add(trackerData);
    }

}
