package mice.cdb.service.parser;

import mice.cdb.service.coolingchannel.CoilData;
import mice.cdb.service.coolingchannel.CoolingchannelConstants;
import mice.cdb.service.coolingchannel.CoolingchannelData;
import mice.cdb.service.coolingchannel.MagnetData;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class CoolingchannelParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private CoolingchannelData m_coolingchannelData = new CoolingchannelData();
    private MagnetData m_magnet = new MagnetData();

    /**
     * Default constructor
     */
    public CoolingchannelParser() {
    }

    /**
     * Get the parsed XML.
     * 
     * @return the parsed XML as a CoolingchannelData
     */
    public CoolingchannelData getData() {
        return m_coolingchannelData;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(CoolingchannelConstants.COOLING_CHANNEL)) {
            // COOLING CHANNEL
            m_coolingchannelData = new CoolingchannelData();

        } else if (qName.equalsIgnoreCase(CoolingchannelConstants.MAGNET)) {
            // MAGNET, part of COOLING CHANNEL
            m_magnet = new MagnetData();
            m_magnet.setName(validateString(attr.getValue(CoolingchannelConstants.NAME)));
            m_magnet.setMode(validateString(attr.getValue(CoolingchannelConstants.MODE)));
            m_magnet.setPolarity(validateString(attr.getValue(CoolingchannelConstants.POLARITY)));
            m_coolingchannelData.addMagnet(m_magnet);

        } else if (qName.equalsIgnoreCase(CoolingchannelConstants.COIL)) {
            // COIL, part of MAGNET
            CoilData coil = new CoilData();
            coil.setName(validateString(attr.getValue(CoolingchannelConstants.NAME)));
            coil.setCalibration(parseDouble(attr.getValue(CoolingchannelConstants.CALIBRATION)));
            coil.setILim(parseDouble(attr.getValue(CoolingchannelConstants.ILIM)));
            coil.setISet(parseDouble(attr.getValue(CoolingchannelConstants.ISET)));
            coil.setRate(parseDouble(attr.getValue(CoolingchannelConstants.RATE)));
            coil.setStability(parseDouble(attr.getValue(CoolingchannelConstants.STABILITY)));
            coil.setVLim(parseDouble(attr.getValue(CoolingchannelConstants.VLIM)));
            m_magnet.addCoil(coil);

        } else if (qName.equalsIgnoreCase(CoolingchannelConstants.TAG)) {
            // TAG
            m_coolingchannelData = new CoolingchannelData();
            m_coolingchannelData.setTag(validateStringNonBlank(attr.getValue(CoolingchannelConstants.NAME)));
        }
    }
}
