package mice.cdb.service.parser;

import java.util.LinkedList;
import java.util.List;

import mice.cdb.service.statemachine.PV;
import mice.cdb.service.statemachine.StateMachineConstants;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class PVParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private List<PV> m_pvList;

    /**
     * Default constructor
     */
    public PVParser() {
    }

    /**
     * Get the PVList containing the parsed XML.
     * 
     * @return the parsed XML as a list of PVs
     */
    public List<PV> getPVList() {
        return m_pvList;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(StateMachineConstants.PV_DATA)) {
            m_pvList = new LinkedList<PV>();
        } else if (qName.equalsIgnoreCase(StateMachineConstants.PV)) {
            addPV(attr);
        }
    }

    private void addPV(Attributes attr) throws SAXException {
        PV pv = new PV();
        pv.setName(validateStringNonBlank(attr.getValue(StateMachineConstants.NAME)));
        pv.setHiHi(parseDouble(attr.getValue(StateMachineConstants.HIHI)));
        pv.setHi(parseDouble(attr.getValue(StateMachineConstants.HI)));
        pv.setLo(parseDouble(attr.getValue(StateMachineConstants.LO)));
        pv.setLoLo(parseDouble(attr.getValue(StateMachineConstants.LOLO)));
        pv.setUnits(validateStringNonBlank(attr.getValue(StateMachineConstants.UNITS)));
        pv.setUnits(validateStringNonBlank(attr.getValue(StateMachineConstants.UNITS)));
        pv.setMode(validateStringNonBlank(attr.getValue(StateMachineConstants.MODE)));
        if (attr.getValue(StateMachineConstants.FREQUENCY) != null) {
            pv.setFrequency(parseDouble(attr.getValue(StateMachineConstants.FREQUENCY)));
        }
        if (attr.getValue(StateMachineConstants.DEADBAND) != null) {
            pv.setDeadband(parseDouble(attr.getValue(StateMachineConstants.DEADBAND)));
        }
        pv.setTransition(parseDouble(attr.getValue(StateMachineConstants.TRANSITION)));
        if (!pv.validate()) {
            throw new SAXException("Error with values of pv data " + pv.getName());
        }
        m_pvList.add(pv);
        m_log.debug("added new PV");
    }
}
