package mice.cdb.service.statemachine;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>StateMachineSuperMouse</code> interface is used to set and retrieve
 * stateMachine data. The state of an individual system can be retrieved using
 * {@link #getCurrentState(String) getCurrentState} and the state of the entire
 * system can be retrieved with {@link #getCurrentStateMachine()
 * getCurrentStateMachine}. Old versions of the data are stored for diagnostic
 * purposes. Data can be retrieved for a given time using
 * {@link #getStateMachineForDate(String) getStateMachineForDate} and for a
 * given run using {@link #getStateMachineForRun(String) getStateMachineForRun}.
 * </p>
 * 
 * <p>
 * The returned XML may contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface StateMachineSuperMouse extends StateMachine {

    /**
     * Set the state of the given system to the given state.
     * 
     * @param system
     *            the system to alter
     * @param state
     *            the state to set
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setState(String system, String state);

    @WebMethod
    public abstract String setPVData(String system, String state, String xml);

}
