package mice.cdb.service.target;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>Target</code> interface is used to retrieve data about target
 * settings. To store target data use the {@link TargetSuperMouse} interface. A
 * number of methods are available to retrieve subsets of the data,
 * {@link #getCurrentBlob(String) getCurrentBlob},
 * {@link #getBlobForRun(String, String) getBlobForRun} and
 * {@link #getBlobForDate(String, String) getBlobForDate}.
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 * 
 */
@WebService
public interface Target {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the current target data for the given target.
     * 
     * @param name
     *            a String representing the name of the target
     * @return a byte array containing the target data set
     * 
     */
    @WebMethod
    public abstract String getCurrentBlob(String name);

    /**
     * Get the target data for the given target and run number.
     * 
     * @param name
     *            a String representing the name of the target
     * @param runNo
     *            the run number
     * @return a byte array containing the target data set
     */
    @WebMethod
    public abstract String getBlobForRun(String name, String runNo);

    /**
     * Get the target data for the given target and time.
     * 
     * @param name
     *            a String representing the name of the target
     * @param dateTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * 
     * @return a byte array containing the target data set
     */
    @WebMethod
    public abstract String getBlobForDate(String name, String dateTime);

    /**
     * Get the list of target names.
     * 
     * @return a list of target names
     */
    @WebMethod
    public abstract List<String> getTargetNames();

    @WebMethod
    public abstract Integer getCurrentUserDelay(String name);

    @WebMethod
    public abstract Integer getCurrentSetBCD(String name);

    // @WebMethod
    // public abstract HashMap<String, Integer> getCurrentTargetIntegers(String
    // name);
    //
    // @WebMethod
    // public abstract HashMap<String, Float> getCurrentTargetFloats(String
    // name);
    //
    // @WebMethod
    // public abstract HashMap<String, String> getCurrentTargetStrings(String
    // name);
}
