package mice.cdb.service.target;

import java.util.HashMap;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>TargetSuperMouse</code> interface is used to store and retrieve
 * data about target settings.A number of methods are available to retrieve
 * subsets of the data, {@link #getCurrentBlob(String) getCurrentBlob},
 * {@link #getBlobForRun(String, String) getBlobForRun} and
 * {@link #getBlobForDate(String, String) getBlobForDate}.
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * The returned XML will contain a message enclosed in one of three tags: ok,
 * warning or error. A tag of warning indicates that the problem maybe transient
 * and retrying the request MAY succeed. A tag of error maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 * 
 */
@WebService
public interface TargetSuperMouse extends Target {

    /**
     * Add a blob of data for a target.
     * 
     * @param name
     *            a String representing the name of the target
     * @param data
     *            a set of target settings
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String addBlob(String name, String data);

    @WebMethod
    public abstract String setUserDelay(String name, Integer time);

    @WebMethod
    public abstract String setSetBCD(String name, Integer distance);

    @WebMethod
    public abstract String setTarget(String name, HashMap<String, Integer> mapInteger, HashMap<String, Float> mapFloat,
            HashMap<String, String> mapString);
}