package mice.cdb.service.alarmhandler;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.sql.Timestamp;
import java.util.regex.Pattern;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class AlarmHandlerSMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static AlarmHandlerSuperMouse m_ah;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(AlarmHandlerSMTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_ah = new AlarmHandlerSuperMouseImpl();
        testClassReady();
    }

    @After
    public void a1_After() {
        endOfTest();
    }

    @AfterClass
    public static void a1_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_ah.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // ListTags()
    @Test
    public final void testListTags() throws Exception {
        String result = m_ah.listTags();
        assertEquals("Tags", "<tags><tag name='Fred'/><tag name='Harry'/></tags>", result);
    }

    // GetTaggedALH(tag)
    @Test
    public final void testGetTaggedALHUnknown() throws Exception {
        String result = m_ah.getTaggedALH("unknown");
        assertEquals("Expecting error message", "<error>No match found for tag = unknown.</error>", result);
    }

    @Test
    public final void testGetTaggedALHKnown() throws Exception {
        String result = m_ah.getTaggedALH("Fred");
        String alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("List of alarms", alarms, result);
    }

    @Test
    public final void testGetTaggedALHBadData() throws Exception {
        getBadTag("u!nknown");
        getBadTag("'");
        getBadTag("\'");
        getBadTag("\"");
        getBadTag("'\"'");
        getBadTag("$");
    }

    @Test
    public final void testGetTaggedALHOKData() throws Exception {
        getAllowedTag("aAZz");
        getAllowedTag("0123456789");
        getAllowedTag("_");
        getAllowedTag("");
        getAllowedTag("-");
        getAllowedTag(":");
        getAllowedTag("+");
        getAllowedTag(".");
        getAllowedTag(",");
    }

    // GetUsedALH(timestamp)
    @Test
    public final void testGetUsedALH() throws Exception {
        String result = m_ah.getUsedALH("2010-10-25 15:29:22.790");
        String alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        result = m_ah.getUsedALH("2010-10-25 15:29:22.791");
        alarms = "<alarmhandler creationtime='2010-10-31 11:05:03.432' tag='Harry'><alarm><name>A1</name><hihi>105.0</hihi><hi>106.0</hi><lo>103.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>11.0</hihi><hi>8.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        result = m_ah.getUsedALH("2010-10-25 15:29:22.792");
        alarms = "<alarmhandler creationtime='2010-10-31 11:05:03.432' tag='Harry'><alarm><name>A1</name><hihi>105.0</hihi><hi>106.0</hi><lo>103.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>11.0</hihi><hi>8.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        String timestamp = new Timestamp(System.currentTimeMillis()).toString();
        result = m_ah.getUsedALH(timestamp);
        alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);
    }

    @Test
    public final void testGetUsedALHTooEarly() throws Exception {
        String result = m_ah.getUsedALH("0200-01-01 00:00:00.999");
        String expectedResponse = "<error>Starttime: 0200-01-01 00:00:00.999 out of range of values in the database.</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    @Test
    public final void testGetUsedALHBadData() throws Exception {
        getBadUsedALH("20.10-10-25 12:03:03.187");
        getBadUsedALH("2010-10-25 12:03:03.18700000Z");
        getBadUsedALH("20A10-10-25 12:03:03.18700000Z");
        getBadUsedALH("20A10-10-25 12:03:03.187'0'00Z");
    }

    // GetUsedTags(startTime, endTime)
    @Test
    public final void testGetUsedTags() throws Exception {
        String tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/>"
                + "<tag starttime='2010-10-25 16:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-26 14:29:22.791' name='Fred'/></tags>";
        String result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2200-10-14 12:32:59.8");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.790");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.791");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.792");
        assertEquals("Tags", tags, result);
    }

    // setTagInUse(tag, timestamp)
    @Test
    public final void testSetTagInUseError() throws Exception {
        String result = m_ah.setTagInUse("Harry", "2010-10-25 12:03:03.187");
        String expectedResponse = "<error>AlarmHandler with tag = Harry and creation time = 2010-10-25 12:03:03.187 does not exist.</error>";
        assertEquals("Expecting error message", expectedResponse, result);

    }

    @Test
    public final void testSetTagInUse() throws Exception {
        String result = m_ah.setTagInUse("Harry", "2010-10-31 11:05:03.432");
        boolean match = false;
        if (Pattern.matches("<ok>Tag Harry set in use at \\d\\d\\d\\d-\\d\\d-\\d\\d \\d\\d:\\d\\d:\\d\\d\\.\\d*</ok>",
                result)) {
            match = true;
        }
        assertTrue("Expecting ok message", match);
    }

    @Test
    public final void testSetTagInUseBadTag() throws Exception {
        setBadTagInUseTag("u!nknown");
        setBadTagInUseTag("'");
        setBadTagInUseTag("\'");
        setBadTagInUseTag("\"");
        setBadTagInUseTag("'\"'");
        setBadTagInUseTag("$");

    }

    @Test
    public final void testSetTagInUseOKTag() throws Exception {
        setAllowedTagInUseTag("aAZz");
        setAllowedTagInUseTag("0123456789");
        setAllowedTagInUseTag("_");
        setAllowedTagInUseTag("");
        setAllowedTagInUseTag("-");
        setAllowedTagInUseTag(":");
        setAllowedTagInUseTag("+");
        setAllowedTagInUseTag(".");
        setAllowedTagInUseTag(",");
    }

    @Test
    public final void testSetTagInUseBadDate() throws Exception {
        setBadTagInUseDate("20.10-10-25 12:03:03.187");
        setBadTagInUseDate("2010-10-25 12:03:03.18700000Z");
        setBadTagInUseDate("20A10-10-25 12:03:03.18700000Z");
        setBadTagInUseDate("20A10-10-25 12:03:03.187'0'00Z");
    }

    // setTagged(tag, xml)
    @Test
    public final void testSetTaggedALH() throws Exception {
        String xml = "<alarmhandler>" + getAlarm("ju1", "10.0", "9.5", "8.5", "8.0")
                + getAlarm("ju2", "1000", "950", "700", "600") + "</alarmhandler>";
        String result = m_ah.setTaggedALH("JUnit", xml);
        String expectedResponse = "<ok>Inserted 2 alarms for tag JUnit</ok>";
        assertEquals("Expecting ok message", expectedResponse, result);

        xml = "<alarmhandler>" + getAlarm("ju1", "0", "0", "0", "0") + "</alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        expectedResponse = "<ok>Inserted 1 alarm for tag JUnit</ok>";
        assertEquals("Expecting ok message", expectedResponse, result);
    }

    @Test
    public final void testSetTaggedALHInvalidValue() throws Exception {
        String xml = "<alarmhandler>" + getAlarm("ju1", "1", "1", "1", "2") + "</alarmhandler>";
        String result = m_ah.setTaggedALH("JUnit", xml);
        String expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. Error with values of alarm ju1</error>";
        assertEquals("Expecting error message", expectedResponse, result);

        xml = "<alarmhandler>" + getAlarm("ju1", "1", "1", "2", "2") + "</alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        assertEquals("Expecting error message", expectedResponse, result);

        xml = "<alarmhandler>" + getAlarm("ju1", "1", "2", "2", "2") + "</alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        assertEquals("Expecting error message", expectedResponse, result);

        // missing name value
        xml = "<alarmhandler>" + getAlarm("", "1", "2", "2", "2") + "</alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. Error with values of alarm </error>";
        assertEquals("Expecting error message", expectedResponse, result);

        // missing hihi value
        xml = "<alarmhandler>" + getAlarm("ju1", "", "2", "2", "2") + "</alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. Could not parse Double:</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    @Test
    public final void testSetTaggedALHInvalidXml() throws Exception {
        // missing hihi element
        String xml = "<alarmhandler><alarm><name>ju1</name><hi>0</hi><lo>0</lo><lolo>0</lolo></alarm></alarmhandler>";
        String result = m_ah.setTaggedALH("JUnit", xml);
        String expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. Error with values of alarm ju1</error>";
        assertEquals("Expecting error message", expectedResponse, result);

        // missing hi element
        xml = "<alarmhandler><alarm><name>ju1</name><hihi>0</hihi><lo>0</lo><lolo>0</lolo></alarm></alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        assertEquals("Expecting error message", expectedResponse, result);

        // missing lo element
        xml = "<alarmhandler><alarm><name>ju1</name><hihi>0</hihi><hi>0</hi><lolo>0</lolo></alarm></alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        assertEquals("Expecting error message", expectedResponse, result);

        // missing lolo element
        xml = "<alarmhandler><alarm><name>ju1</name><hihi>0</hihi><hi>0</hi><lolo>0</lolo></alarm></alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        assertEquals("Expecting error message", expectedResponse, result);

        // missing name element
        xml = "<alarmhandler><alarm><hihi>5</hihi><hi>4</hi>3<lo>2</lo><lolo>1</lolo></alarm></alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. Error with values of alarm </error>";
        assertEquals("Expecting error message", expectedResponse, result);

        // missing </alarm>
        xml = "<alarmhandler><alarm><name>ju1</name><hihi>5</hihi><hi>4</hi>3<lo>2</lo><lolo>1</lolo></alarmhandler>";
        result = m_ah.setTaggedALH("JUnit", xml);
        expectedResponse = "<error>Could not parse AlarmHandler XML. Please check input. The end-tag for element type \"alarm\" must end with a '>' delimiter.</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    @Test
    public final void testSetTaggedALHBadTag() throws Exception {
        setBadTag("u!nknown");
        setBadTag("'");
        setBadTag("\'");
        setBadTag("\"");
        setBadTag("'\"'");
        setBadTag("$");
    }

    @Test
    public final void testSetTaggedALHOKTag() throws Exception {
        setAllowedTag("aAZz");
        setAllowedTag("0123456789");
        setAllowedTag("_");
        setAllowedTag("");
        setAllowedTag("-");
        setAllowedTag("+");
        setAllowedTag(":");
        setAllowedTag(".");
    }

    // setTagged(tag, xml), getTaggedALH(tag)
    @Test
    public final void testSetGetTaggedALH() throws Exception {
        String tag = "testSetGetTaggedALH";
        String xml = "<alarmhandler>" + getAlarm(tag + "1", "10.0", "9.5", "8.5", "8.0") + "</alarmhandler>";
        String result = m_ah.setTaggedALH(tag, xml);
        String expectedResponse = "<ok>Inserted 1 alarm for tag " + tag + "</ok>";
        assertEquals("Expecting ok message", expectedResponse, result);

        result = m_ah.getTaggedALH(tag);
        boolean match = false;
        expectedResponse = "<alarmhandler creationtime='\\d\\d\\d\\d-\\d\\d-\\d\\d \\d\\d:\\d\\d:\\d\\d\\.\\d*' tag='testSetGetTaggedALH'><alarm><name>testSetGetTaggedALH1</name><hihi>10.0</hihi><hi>9.5</hi><lo>8.5</lo><lolo>8.0</lolo></alarm></alarmhandler>";
        if (Pattern.matches(expectedResponse, result)) {
            match = true;
        }
        assertTrue("Expecting ok message", match);
    }

    // setTagInUse(tag, creationTime),
    @Test
    public final void testSetGetTag() throws Exception {
        String tag1 = "testSetGetTag1";
        String xml = "<alarmhandler>" + getAlarm(tag1 + "a", "10.0", "9.5", "8.5", "8.0") + "</alarmhandler>";
        String result = m_ah.setTaggedALH(tag1, xml);
        result = m_ah.getTaggedALH(tag1);
        String creationTime1 = result.split("'")[1];

        String tag2 = "testSetGetTag2";
        xml = "<alarmhandler>" + getAlarm(tag2 + "b", "10.0", "9.5", "8.5", "8.0") + "</alarmhandler>";
        result = m_ah.setTaggedALH(tag2, xml);
        result = m_ah.getTaggedALH(tag2);
        String creationTime2 = result.split("'")[1];

        Thread.sleep(100);
        Timestamp timestamp0 = new Timestamp(System.currentTimeMillis());
        Thread.sleep(100);
        result = m_ah.setTagInUse(tag1, creationTime1);
        Thread.sleep(100);
        Timestamp timestamp1 = new Timestamp(System.currentTimeMillis());
        Thread.sleep(100);
        Timestamp timestamp1a = new Timestamp(System.currentTimeMillis());
        Thread.sleep(100);
        result = m_ah.setTagInUse(tag2, creationTime2);
        Thread.sleep(100);
        Timestamp timestamp2 = new Timestamp(System.currentTimeMillis());

        result = m_ah.getUsedTags(timestamp1.toString(), timestamp1a.toString());
        boolean match = false;
        String expectedResponse = "<tags><tag starttime='\\d\\d\\d\\d-\\d\\d-\\d\\d \\d\\d:\\d\\d:\\d\\d\\.\\d*' name='"
                + tag1 + "'/></tags>";
        if (Pattern.matches(expectedResponse, result)) {
            match = true;
        }
        assertTrue("tags do not match " + result, match);
        Timestamp starttime1 = Timestamp.valueOf(result.split("'")[1]);
        match = false;
        if (timestamp0.compareTo(starttime1) < 0 & timestamp1.compareTo(starttime1) > 0) {
            match = true;
        }
        assertTrue("starttime is wrong " + result, match);

        result = m_ah.getUsedTags(timestamp1.toString(), timestamp2.toString());
        match = false;
        expectedResponse = "<tags><tag starttime='\\d\\d\\d\\d-\\d\\d-\\d\\d \\d\\d:\\d\\d:\\d\\d\\.\\d*' name='"
                + tag1 + "'/><tag starttime='\\d\\d\\d\\d-\\d\\d-\\d\\d \\d\\d:\\d\\d:\\d\\d\\.\\d*' name='" + tag2
                + "'/></tags>";
        if (Pattern.matches(expectedResponse, result)) {
            match = true;
        }
        assertTrue("tags do not match " + result, match);

        result = m_ah.getUsedALH(timestamp1.toString());
        match = false;
        expectedResponse = "<alarmhandler creationtime='" + creationTime1 + "' tag='" + tag1 + "'><alarm><name>" + tag1
                + "a</name><hihi>10.0</hihi><hi>9.5</hi><lo>8.5</lo><lolo>8.0</lolo></alarm></alarmhandler>";
        if (Pattern.matches(expectedResponse, result)) {
            match = true;
        }
        assertTrue("tags do not match " + result, match);
    }

    private void getBadTag(String tag) throws Exception {
        String result = m_ah.getTaggedALH(tag);
        assertEquals("Expecting error message", "<error>String contained non-aphanumeric/non-white space characters: "
                + tag + "</error>", result);
    }

    private void getAllowedTag(String tag) throws Exception {
        String result = m_ah.getTaggedALH(tag);
        assertEquals("Expecting error message", "<error>No match found for tag = " + tag + ".</error>", result);
    }

    public void getBadUsedALH(String timestamp) throws Exception {
        String result = m_ah.getUsedALH(timestamp);
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    private String getAlarm(String name, String hihi, String hi, String lo, String lolo) {
        return "<alarm><name>" + name + "</name><hihi>" + hihi + "</hihi><hi>" + hi + "</hi>" + "<lo>" + lo
                + "</lo><lolo>" + lolo + "</lolo></alarm>";
    }

    private void setBadTagInUseTag(String tag) throws Exception {
        String result = m_ah.setTagInUse(tag, "2010-10-31 11:05:03.432");
        assertEquals("Expecting error message", "<error>String contained non-aphanumeric/non-white space characters: "
                + tag + "</error>", result);
    }

    private void setAllowedTagInUseTag(String tag) throws Exception {
        String result = m_ah.setTagInUse(tag, "2010-10-31 11:05:03.432");
        assertEquals("Expecting error message", "<error>AlarmHandler with tag = " + tag
                + " and creation time = 2010-10-31 11:05:03.432 does not exist.</error>", result);
    }

    public void setBadTagInUseDate(String timestamp) throws Exception {
        String result = m_ah.setTagInUse("Harry", timestamp);
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    private void setBadTag(String tag) throws Exception {
        String xml = "<alarmhandler>" + getAlarm("ju1", "0", "0", "0", "0") + "</alarmhandler>";
        String result = m_ah.setTaggedALH(tag, xml);
        assertEquals("Expecting error message", "<error>String contained non-aphanumeric/non-white space characters: "
                + tag + "</error>", result);
    }

    private void setAllowedTag(String tag) throws Exception {
        String xml = "<alarmhandler>" + getAlarm("ju1", "0", "0", "0", "0") + "</alarmhandler>";
        String result = m_ah.setTaggedALH(tag, xml);
        assertEquals("Expecting ok message", "<ok>Inserted 1 alarm for tag " + tag + "</ok>", result);
    }
}
