package mice.cdb.service.alarmhandler;

import static org.junit.Assert.assertEquals;

import java.sql.Timestamp;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class AlarmHandlerTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static AlarmHandler m_ah;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(AlarmHandlerTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_ah = new AlarmHandlerImpl();
        testClassReady();
    }

    @After
    public void a1_After() {
        endOfTest();
    }

    @AfterClass
    public static void a1_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_ah.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // ListTags()
    @Test
    public final void testListTags() throws Exception {
        String result = m_ah.listTags();
        assertEquals("Tags", "<tags><tag name='Fred'/><tag name='Harry'/></tags>", result);
    }

    // GetTaggedALH(tag)
    @Test
    public final void testGetTaggedALHUnknown() throws Exception {
        String result = m_ah.getTaggedALH("unknown");
        assertEquals("Expecting error message", "<error>No match found for tag = unknown.</error>", result);
    }

    @Test
    public final void testGetTaggedALHKnown() throws Exception {
        String result = m_ah.getTaggedALH("Fred");
        String alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("List of alarms", alarms, result);
    }

    @Test
    public final void testGetTaggedALHBadData() throws Exception {
        getBadTag("u!nknown");
        getBadTag("'");
        getBadTag("\'");
        getBadTag("\"");
        getBadTag("'\"'");
        getBadTag("$");
    }

    @Test
    public final void testGetTaggedALHOKData() throws Exception {
        getAllowedTag("aAZz");
        getAllowedTag("0123456789");
        getAllowedTag("_");
        getAllowedTag("");
        getAllowedTag("-");
        getAllowedTag(":");
        getAllowedTag("+");
        getAllowedTag(".");
        getAllowedTag(",");
    }

    // GetUsedALH(timestamp)
    @Test
    public final void testGetUsedALH() throws Exception {
        String result = m_ah.getUsedALH("2010-10-25 15:29:22.790");
        String alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        result = m_ah.getUsedALH("2010-10-25 15:29:22.791");
        alarms = "<alarmhandler creationtime='2010-10-31 11:05:03.432' tag='Harry'><alarm><name>A1</name><hihi>105.0</hihi><hi>106.0</hi><lo>103.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>11.0</hihi><hi>8.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        result = m_ah.getUsedALH("2010-10-25 15:29:22.792");
        alarms = "<alarmhandler creationtime='2010-10-31 11:05:03.432' tag='Harry'><alarm><name>A1</name><hihi>105.0</hihi><hi>106.0</hi><lo>103.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>11.0</hihi><hi>8.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);

        String timestamp = new Timestamp(System.currentTimeMillis()).toString();
        result = m_ah.getUsedALH(timestamp);
        alarms = "<alarmhandler creationtime='2010-10-25 12:03:03.187' tag='Fred'><alarm><name>A1</name><hihi>104.0</hihi><hi>105.0</hi><lo>102.0</lo><lolo>101.0</lolo></alarm><alarm><name>A2</name><hihi>10.0</hihi><hi>9.0</hi><lo>80.0</lo><lolo>7.0</lolo></alarm></alarmhandler>";
        assertEquals("Alarms", alarms, result);
    }

    @Test
    public final void testGetUsedALHTooEarly() throws Exception {
        String result = m_ah.getUsedALH("0200-01-01 00:00:00.999");
        String expectedResponse = "<error>Starttime: 0200-01-01 00:00:00.999 out of range of values in the database.</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

    @Test
    public final void testGetUsedALHBadData() throws Exception {
        getBadUsedALH("20.10-10-25 12:03:03.187");
        getBadUsedALH("2010-10-25 12:03:03.18700000Z");
        getBadUsedALH("20A10-10-25 12:03:03.18700000Z");
        getBadUsedALH("20A10-10-25 12:03:03.187'0'00Z");
    }

    // GetUsedTags(startTime, endTime)
    @Test
    public final void testGetUsedTags() throws Exception {
        String tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/>"
                + "<tag starttime='2010-10-25 16:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-26 14:29:22.791' name='Fred'/></tags>";
        String result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2200-10-14 12:32:59.8");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.790");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.791");
        assertEquals("Tags", tags, result);

        tags = "<tags><tag starttime='2010-10-25 14:29:22.791' name='Fred'/>"
                + "<tag starttime='2010-10-25 15:29:22.791' name='Harry'/></tags>";
        result = m_ah.getUsedTags("2000-10-14 12:32:59.8", "2010-10-25 15:29:22.792");
        assertEquals("Tags", tags, result);
    }

    private void getBadTag(String tag) throws Exception {
        String result = m_ah.getTaggedALH(tag);
        assertEquals("Expecting error message", "<error>String contained non-aphanumeric/non-white space characters: "
                + tag + "</error>", result);
    }

    private void getAllowedTag(String tag) throws Exception {
        String result = m_ah.getTaggedALH(tag);
        assertEquals("List of alarms", "<error>No match found for tag = " + tag + ".</error>", result);
    }

    private void getBadUsedALH(String timestamp) throws Exception {
        String result = m_ah.getUsedALH(timestamp);
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

}
