package mice.cdb.service.beamline;

import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class BeamlineSMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static BeamlineSuperMouse m_beamline;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(BeamlineSMTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_beamline = new BeamlineSuperMouseImpl();
        testClassReady();
    }

    @After
    public void beamline_After() {
        endOfTest();
    }

    @AfterClass
    public static void beamline_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_beamline.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getAllBeamlines()
    @Test
    public final void testGetAllBeamlines() throws Exception {
        String result = m_beamline.getAllBeamlines();
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run><run runNumber='2' startTime='2010-10-26 16:29:22.791' endTime='2010-10-26 16:39:22.791' startNotes='junk' endNotes='endjunk2' optics='opts' protonAbsorberThickness='22' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-2' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='false' diffuserThickness='21' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='6'/></run><run runNumber='3' startTime='2010-10-28 16:29:22.791' endTime='2010-10-28 16:39:22.791' startNotes='junk' endNotes='endjunk3' optics='opts' protonAbsorberThickness='32' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='null' daqGateWidth='null' daqVersion='null' beamStop='true' diffuserThickness='31' gdcHostName='null' targetDelay='null' targetDepth='null' targetDriveVoltage='null'></run></runs>";
        assertEquals("runs", expectedResponse, result);

    }

    // getBeamlineForRun(runNumber)
    @Test
    public final void testGetBeamlineForRun() throws Exception {
        String result = m_beamline.getBeamlineForRun("1");
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run><run runNumber='2' startTime='2010-10-26 16:29:22.791' endTime='2010-10-26 16:39:22.791' startNotes='junk' endNotes='endjunk2' optics='opts' protonAbsorberThickness='22' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-2' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='false' diffuserThickness='21' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='6'/></run><run runNumber='3' startTime='2010-10-28 16:29:22.791' endTime='2010-10-28 16:39:22.791' startNotes='junk' endNotes='endjunk3' optics='opts' protonAbsorberThickness='32' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='null' daqGateWidth='null' daqVersion='null' beamStop='true' diffuserThickness='31' gdcHostName='null' targetDelay='null' targetDepth='null' targetDriveVoltage='null'></run></runs>";

        result = m_beamline.getBeamlineForRun("99");
        expectedResponse = "<runs></runs>";
        assertEquals("run", expectedResponse, result);
    }

    @Test
    public final void testGetBeamlineForRunBadData() throws Exception {
        String result = m_beamline.getBeamlineForRun("1a");
        String expectedResponse = "<error>Value:1a is not a valid int</error>";
        assertEquals("run", expectedResponse, result);
    }

    // getBeamlinesForDates(timestamp, timestamp)
    @Test
    public final void testGetBeamlinesForDates() throws Exception {
        getNoBeamlineForDates("2010-10-25 16:29:22.789", "2010-10-25 16:29:22.790");
        getOneBeamlineForDates("2010-10-25 16:29:22.790", "2010-10-25 16:29:22.791");
        getOneBeamlineForDates("2010-10-25 16:29:22.790", "2010-10-25 16:29:22.792");
        getOneBeamlineForDates("2010-10-25 16:29:22.790", "2010-10-25 16:39:22.791");
        getOneBeamlineForDates("2010-10-25 16:29:22.790", "2010-10-25 16:39:22.792");
        getOneBeamlineForDates("2010-10-25 16:29:22.791", "2010-10-25 16:39:22.790");
        getOneBeamlineForDates("2010-10-25 16:29:22.791", "2010-10-25 16:39:22.791");
        getOneBeamlineForDates("2010-10-25 16:29:22.791", "2010-10-25 16:39:22.791");
        getOneBeamlineForDates("2010-10-25 16:29:22.792", "2010-10-25 16:39:22.790");
        getOneBeamlineForDates("2010-10-25 16:29:22.792", "2010-10-25 16:39:22.791");
        getOneBeamlineForDates("2010-10-25 16:29:22.792", "2010-10-25 16:39:22.792");
        getOneBeamlineForDates("2010-10-25 16:39:22.791", "2010-10-25 16:39:22.792");
        getNoBeamlineForDates("2010-10-25 16:39:22.792", "2010-10-25 16:39:22.792");
        String result = m_beamline.getBeamlinesForDates("2010-10-25 16:39:22.791", "2010-10-26 16:29:22.791");
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run><run runNumber='2' startTime='2010-10-26 16:29:22.791' endTime='2010-10-26 16:39:22.791' startNotes='junk' endNotes='endjunk2' optics='opts' protonAbsorberThickness='22' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-2' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='false' diffuserThickness='21' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='6'/></run></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testGetBeamlinesForDatesBadData() throws Exception {
        getBadBeamlinesForDates("20.10-10-25 12:03:03.187");
        getBadBeamlinesForDates("2010-10-25 12:03:03.18700000Z");
        getBadBeamlinesForDates("20A10-10-25 12:03:03.18700000Z");
        getBadBeamlinesForDates("20A10-10-25 12:03:03.187'0'00Z");
    }

    // getBeamlinesForPulses(startPulse, endPulse)
    @Test
    public final void testGetBeamlinesForPulses() throws Exception {
        getNoBeamlineForPulses("123", "542");
        getOneBeamlineForPulses("542", "543");
        getOneBeamlineForPulses("542", "544");
        getOneBeamlineForPulses("542", "555");
        getOneBeamlineForPulses("542", "556");
        getOneBeamlineForPulses("543", "554");
        getOneBeamlineForPulses("543", "555");
        getOneBeamlineForPulses("543", "556");
        getOneBeamlineForPulses("544", "554");
        getOneBeamlineForPulses("544", "555");
        getOneBeamlineForPulses("544", "556");
        getOneBeamlineForPulses("555", "556");
        getNoBeamlineForPulses("556", "556");
        String result = m_beamline.getBeamlinesForPulses("555", "666");
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run><run runNumber='2' startTime='2010-10-26 16:29:22.791' endTime='2010-10-26 16:39:22.791' startNotes='junk' endNotes='endjunk2' optics='opts' protonAbsorberThickness='22' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-2' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='false' diffuserThickness='21' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='6'/></run><run runNumber='3' startTime='2010-10-28 16:29:22.791' endTime='2010-10-28 16:39:22.791' startNotes='junk' endNotes='endjunk3' optics='opts' protonAbsorberThickness='32' startPulse='666' endPulse='777' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='null' daqGateWidth='null' daqVersion='null' beamStop='true' diffuserThickness='31' gdcHostName='null' targetDelay='null' targetDepth='null' targetDriveVoltage='null'></run></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testGetBeamlinesForPulsesBadData() throws Exception {
        getBadBeamlinesForPulses("a", "1", "a");
        getBadBeamlinesForPulses("1", "a", "a");
        getBadBeamlinesForPulses("a", "a", "a");
        getBadBeamlinesForPulses("1.1", "1", "1.1");
        getBadBeamlinesForPulses("1", "1.1", "1.1");
        getBadBeamlinesForPulses("1.1", "1.1", "1.1");
    }

    // setStartRun(xml)
    @Test
    public final void testAddRun() throws Exception {
        String xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        String result = m_beamline.setStartRun(xml);
        String expectedResponse = "<ok>Run data added for run 4</ok>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testAddRunDuplicate() throws Exception {
        String xml = "<startRun runNumber='2' startTime='2010-10-25 16:29:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        String result = m_beamline.setStartRun(xml);
        String expectedResponse = "<error>Problem inserting data into the database - duplicate record. No data has been inserted.</error>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testAddRunBadData() throws Exception {
        String xml = "<startRun runNumber='a' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        String result = m_beamline.setStartRun(xml);
        String expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:a</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='201a-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='j!unk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. String contained non-aphanumeric/non-white space characters: j!unk</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='op\\\"ts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. String contained non-aphanumeric/non-white space characters: op\\\"ts</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='54.3' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:54.3</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Double:1.1.5</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='0\\\"01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. String contained non-aphanumeric/non-white space characters: 0\\\"01011110101111000101111010111100</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF$1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. String contained non-aphanumeric/non-white space characters: TOF$1</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4i' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Double:7.4i</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1$' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. String contained non-aphanumeric/non-white space characters: q1$</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9i' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Double:5.9i</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. XML document structures must start and end within the same entity.</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:null</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<startRun runNumber='4' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        result = m_beamline.setStartRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Double:null</error>";
        assertEquals("runs", expectedResponse, result);
    }

    // setEndRun(xml)
    @Test
    public final void testUpdateRun() throws Exception {
        String xml = "<startRun runNumber='5' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        m_beamline.setStartRun(xml);
        xml = "<endRun runNumber='5' endTime='2010-10-25 16:29:22.791' endPulse='543' status='true'/>";
        String result = m_beamline.setEndRun(xml);
        String expectedResponse = "<ok>Run data updated for run 5</ok>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testUpdateRunDuplicate() throws Exception {
        String xml = "<startRun runNumber='6' startTime='2010-10-25 16:29:22.791' startNotes='junk' optics='opts' protonAbsorberThickness='12' startPulse='543' step='1.5' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/></startRun>";
        m_beamline.setStartRun(xml);
        xml = "<endRun runNumber='6' endTime='2010-10-25 16:29:22.791' endPulse='543' status='true'/>";
        m_beamline.setEndRun(xml);
        xml = "<endRun runNumber='6' endTime='2010-10-25 16:29:22.791' endPulse='543' status='0'/>";
        String result = m_beamline.setEndRun(xml);
        String expectedResponse = "<error>Entry for run 6 does not exist or already has the end data set</error>";
        assertEquals("runs", expectedResponse, result);
    }

    @Test
    public final void testUpdateRunBadData() throws Exception {
        String xml = "<endRun runNumber='a' endTime='2010-10-25 16:29:22.791' endPulse='543' status='0'/>";
        String result = m_beamline.setEndRun(xml);
        String expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:a</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endTime='201I-10-25 16:29:22.791' endPulse='543' status='0'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endTime='2010-10-25 16:29:22.791' endPulse='54.3' status='0'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:54.3</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endTime='2010-10-25 16:29:22.791' endPulse='543' status='2'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse boolean:2</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun endTime='2010-10-25 16:29:22.791' endPulse='543' status='0'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:null</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endPulse='543' status='0'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endTime='2010-10-25 16:29:22.791' status='0'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse Int:null</error>";
        assertEquals("runs", expectedResponse, result);

        xml = "<endRun runNumber='3' endTime='2010-10-25 16:29:22.791' endPulse='543'/>";
        result = m_beamline.setEndRun(xml);
        expectedResponse = "<error>Could not parse Beamline XML. Please check input. Could not parse boolean:null</error>";
        assertEquals("runs", expectedResponse, result);
    }

    private void getOneBeamlineForDates(String start, String end) throws Exception {
        String result = m_beamline.getBeamlinesForDates(start, end);
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    private void getNoBeamlineForDates(String start, String end) throws Exception {
        String result = m_beamline.getBeamlinesForDates(start, end);
        String expectedResponse = "<runs></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    private void getBadBeamlinesForDates(String timestamp) throws Exception {
        String result = m_beamline.getBeamlinesForDates(timestamp, timestamp);
        String message = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", message, result);
    }

    private void getOneBeamlineForPulses(String start, String end) throws Exception {
        String result = m_beamline.getBeamlinesForPulses(start, end);
        String expectedResponse = "<runs><run runNumber='1' startTime='2010-10-25 16:29:22.791' endTime='2010-10-25 16:39:22.791' startNotes='junk' endNotes='endjunk1' optics='opts' protonAbsorberThickness='12' startPulse='543' endPulse='555' step='1.5' status='true' runType='01011110101111000101111010111100' daqTrigger='TOF-1' daqGateWidth='7.4' daqVersion='3.2.1-42' beamStop='true' diffuserThickness='11' gdcHostName='null' targetDelay='42.1' targetDepth='9.8' targetDriveVoltage='240.9'><magnet name='q1' setCurrent='5.9' polarity='1'/><magnet name='q2' setCurrent='7.54' polarity='-1'/><scalar name='SumPartTrig' value='1'/><scalar name='SumReqTrig' value='2'/></run></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    private void getNoBeamlineForPulses(String start, String end) throws Exception {
        String result = m_beamline.getBeamlinesForPulses(start, end);
        String expectedResponse = "<runs></runs>";
        assertEquals("runs", expectedResponse, result);
    }

    private void getBadBeamlinesForPulses(String start, String end, String error) throws Exception {
        String result = m_beamline.getBeamlinesForPulses(start, end);
        String message = "<error>Value:" + error + " is not a valid int</error>";
        assertEquals("Expecting error message", message, result);
    }

}
