package mice.cdb.service.cabling;

import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class CablingSMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static CablingSuperMouse m_cabling;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(CablingSMTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_cabling = new CablingSuperMouseImpl();
        testClassReady();
    }

    @After
    public void beamline_After() {
        endOfTest();
    }

    @AfterClass
    public static void beamline_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_cabling.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getCablingForDate(device, timestamp)
    @Test
    public final void testGetCablingForDateConntrol() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='CONTROL'></cabling>";
        getCablingForDate("Control", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'>"
                + "<module number='1'><channel number='1' name='c11'/><channel number='2' name='c12'/></module>"
                + "<module number='2'><channel number='1' name='c13'/></module></crate>" + "<crate number='2'>"
                + "<module number='3'><channel number='1' name='c14'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.792", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCablingForDateDetector() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='DETECTOR'></cabling>";
        getCablingForDate("Ckov A", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCablingForDateTracker() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='TRACKER'></cabling>";
        getCablingForDate("Tracker 1", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='110'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='141' station='151' plane='161' channel='171'/></vlsbGeoNumber><vlsbGeoNumber number='121'><vlsbChannel number='130' trackerNumber='142' station='152' plane='162' channel='172'/><vlsbChannel number='131' trackerNumber='143' station='153' plane='163' channel='173'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='111'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='144' station='154' plane='164' channel='174'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2010-10-27 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2010-10-27 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2011-10-27 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCablingForDateError() throws Exception {
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        getCablingForDate("Junk", "20c10-10-25 16:29:22.789", expectedResponse);
        getCablingForDate("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: Ju'nk</error>";
        getCablingForDate("Ju'nk", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCablingForDate("", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCablingForDate("Junk", "2010-10-25 16:29:22.789", expectedResponse);
    }

    private void getCablingForDate(String device, String timestamp, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCablingForDate(device, timestamp));
    }

    // getCablingForRun(device, runNo)
    @Test
    public final void testGetCablingForRunControl() throws Exception {
        String expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c11'/><channel number='2' name='c12'/></module><module number='2'><channel number='1' name='c13'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c14'/></module></crate></cabling>";
        getCablingForRun("control", "2", expectedResponse);
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForRun("control", "3", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCablingForRunDetector() throws Exception {
        String expectedResponse = ""; // TODO
        getCablingForRun("Ckov A", "2", expectedResponse);
        expectedResponse = ""; // TODO
        getCablingForRun("Ckov A", "3", expectedResponse);
    }

    @Test
    public final void testGetCablingForRunTracker() throws Exception {
        String expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='110'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='141' station='151' plane='161' channel='171'/></vlsbGeoNumber><vlsbGeoNumber number='121'><vlsbChannel number='130' trackerNumber='142' station='152' plane='162' channel='172'/><vlsbChannel number='131' trackerNumber='143' station='153' plane='163' channel='173'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='111'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='144' station='154' plane='164' channel='174'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForRun("Tracker 1", "2", expectedResponse);
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForRun("Tracker 1", "3", expectedResponse);
    }

    @Test
    public final void testGetCablingForRunError() throws Exception {
        String expectedResponse = "<error>Run 0 not found.</error>";
        getCablingForRun("Junk", "0", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        getCablingForRun("Junk", "a", expectedResponse);
        expectedResponse = "<error>Value: is not a valid int</error>";
        getCablingForRun("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: J'unk</error>";
        getCablingForRun("J'unk", "1", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCablingForRun("", "1", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCablingForRun("Junk", "1", expectedResponse);
    }

    private void getCablingForRun(String device, String runNo, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCablingForRun(device, runNo));
    }

    // getCurrentCabling(device)
    @Test
    public final void testGetCurrentCablingControl() throws Exception {
        String expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCurrentCabling("control", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCurrentCablingDetector() throws Exception {
        String expectedResponse = "Ckov b"; // TODO
        getCurrentCabling("", expectedResponse);
    }

    @Test
    public final void testGetCurrentCablingTracker() throws Exception {
        String expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCurrentCabling("Tracker 1", expectedResponse);
    }

    @Test
    public final void testGetCurrentCablingError() throws Exception {
        String expectedResponse = "<error>Unknown device:Junk</error>";
        getCurrentCabling("Junk", expectedResponse);
    }

    private void getCurrentCabling(String device, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCurrentCabling(device));
    }

    // listDevices()
    @Test
    public final void testListDevices() throws Exception {
        String result = m_cabling.listDevices();
        String expectedResponse = "<cabling><deviceList><device name='CONTROL'/>"
                + "<device name='CKOV A'/><device name='CKOV B'/><device name='EMR'/><device name='KL'/><device name='SCALERS'/>"
                + "<device name='TOF0'/><device name='TOF1'/><device name='TOF2'/><device name='TRIGGER'/><device name='TRIGGER REQUEST'/>"
                + "<device name='TRACKER 1'/><device name='TRACKER 2'/></deviceList></cabling>";
        assertEquals("cabling", expectedResponse, result);
    }

    // addControl(crate, module, channel, name)
    @Test
    public final void testAddControl() throws Exception {
        String expectedResponse = "<ok>Control cabling added</ok>";
        addControl("8", "1", "1", "A1", expectedResponse);
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate><crate number='8'><module number='1'><channel number='1' name='A1'/></module></crate></cabling>";
        getCurrentCabling("control", expectedResponse);
    }

    @Test
    public final void testAddControlError() throws Exception {
        String expectedResponse = "<error>Entry for crate:8' module:1, channel:1 already exists</error>";
        addControl("8", "1", "1", "A1", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        addControl("a", "1", "1", "A1", expectedResponse);
        addControl("9", "a", "1", "A1", expectedResponse);
        addControl("9", "1", "a", "A1", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: A'1</error>";
        addControl("9", "1", "1", "A'1", expectedResponse);
    }

    // updateControl(crate, module, channel, name)
    @Test
    public final void testUpdateControl() throws Exception {
        String expectedResponse = "<ok>Control cabling updated</ok>";
        updateControl("8", "1", "1", "A2", expectedResponse);
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate><crate number='8'><module number='1'><channel number='1' name='A2'/></module></crate></cabling>";
        getCurrentCabling("control", expectedResponse);
    }

    @Test
    public final void testUpdateControlError() throws Exception {
        String expectedResponse = "<error>Value:a is not a valid int</error>";
        updateControl("a", "1", "1", "A2", expectedResponse);
        updateControl("9", "a", "1", "A2", expectedResponse);
        updateControl("9", "1", "a", "A2", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: A'2</error>";
        updateControl("9", "1", "1", "A'2", expectedResponse);
    }

    // addDetector(name, vlsbComputerId, vlsbGeoNumber, vlsbChannel, detectorNo,
    // station, plane, channel)
    @Ignore
    @Test
    public final void testAddDetector() throws Exception {
        String expectedResponse = "<ok>Detector cabling added</ok>";
        addDetector("Detector 1", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("detector 1", "1", "2", "31", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        expectedResponse = ""; // TODO
        getCurrentCabling("detector 1", expectedResponse);
    }

    @Ignore
    @Test
    public final void testAddDetectorError() throws Exception {
        String expectedResponse = ""; // TODO
        addDetector("Detector 1", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        expectedResponse = "<error>Unknown detector:TRACKER 01</error>";
        addDetector("Detector 01", "1", "2", "32", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        addDetector("Detector 2", "a", "2", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "a", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "a", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "a", "5", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "a", "6", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "5", "a", "7", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "5", "6", "a", "8", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "a", "9", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "8", "a", "10", expectedResponse);
        addDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "8", "9", "aa", expectedResponse);
    }

    // updateDetector(name, vlsbComputerId, vlsbGeoNumber, vlsbChannel,
    // detectorNo, station, plane, channel)
    @Ignore
    @Test
    public final void testUpdateDetector() throws Exception {
        String expectedResponse = "<ok>Detector cabling updated</ok>";
        updateDetector("Detector 1", "1", "2", "3", "4", "5", "6", "71", "8", "9", "10", expectedResponse);
        updateDetector("detector 1", "1", "2", "31", "4", "5", "6", "72", "8", "9", "10", expectedResponse);
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='1'><vlsbGeoNumber number='2'><vlsbChannel number='3' detectorNumber='4' station='5' plane='6' channel='71'/><vlsbChannel number='31' detectorNumber='4' station='5' plane='6' channel='72'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCurrentCabling("detector 1", expectedResponse);
    }

    @Ignore
    @Test
    public final void testUpdateDetectorError() throws Exception {
        String expectedResponse = "<error>Unknown detector:TRACKER 01</error>";
        addDetector("Detector 01", "1", "2", "32", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        updateDetector("Detector 2", "a", "2", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "a", "3", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "a", "4", "5", "6", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "a", "5", "6", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "a", "6", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "5", "a", "7", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "5", "6", "a", "8", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "a", "9", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "8", "a", "10", expectedResponse);
        updateDetector("Detector 2", "1", "2", "3", "4", "5", "6", "7", "8", "9", "aa", expectedResponse);
    }

    @Test
    public final void testSetTracker() throws Exception {
        String expectedResponse = "<ok>Tracker cabling set</ok>";
        setTracker("Tracker 2", "1", "2", "3", "4", "5", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "31", "4", "5", "6", "7", expectedResponse);
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='1'><vlsbGeoNumber number='2'><vlsbChannel number='3' trackerNumber='4' station='5' plane='6' channel='7'/><vlsbChannel number='31' trackerNumber='4' station='5' plane='6' channel='7'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCurrentCabling("Tracker 2", expectedResponse);
    }

    @Test
    public final void testSetTrackerError() throws Exception {
        String expectedResponse = "<error>Unknown tracker:TRACKER 01</error>";
        setTracker("Tracker 01", "1", "2", "32", "4", "5", "6", "7", expectedResponse);
        expectedResponse = "<error>Could not parse Cabling XML. Please check input. Could not parse Int:a</error>";
        setTracker("Tracker 2", "a", "2", "3", "4", "5", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "a", "3", "4", "5", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "a", "4", "5", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "3", "a", "5", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "3", "4", "a", "6", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "3", "4", "5", "a", "7", expectedResponse);
        setTracker("Tracker 2", "1", "2", "3", "4", "5", "6", "a", expectedResponse);
    }

    @Test
    public final void testUpdateTracker() throws Exception {
        String expectedResponse = "<ok>Tracker cabling set</ok>";
        setTracker("Tracker 2", "1", "2", "3", "4", "5", "6", "71", expectedResponse);
        setTracker("Tracker 2", "1", "2", "31", "4", "5", "6", "72", expectedResponse);
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='1'><vlsbGeoNumber number='2'><vlsbChannel number='3' trackerNumber='4' station='5' plane='6' channel='71'/><vlsbChannel number='31' trackerNumber='4' station='5' plane='6' channel='72'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCurrentCabling("Tracker 2", expectedResponse);
    }

    private void addControl(String crate, String module, String channel, String name, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.addControl(crate, module, channel, name));
    }

    private void updateControl(String crate, String module, String channel, String name, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.updateControl(crate, module, channel, name));
    }

    private void addDetector(String name, String adcComputerId, String adcGeoNumber, String adcChannel,
            String tdcComputerId, String tdcGeoNumber, String tdcChannel, String station, String plane, String slab,
            String pmt, String expectedResponse) {
        // assertEquals("cabling", expectedResponse, m_cabling.setDetector(name,
        // adcComputerId, adcGeoNumber, adcChannel,
        // tdcComputerId, tdcGeoNumber, tdcChannel, station, plane, slab, pmt));
        // //TODO
    }

    private void updateDetector(String name, String adcComputerId, String adcGeoNumber, String adcChannel,
            String tdcComputerId, String tdcGeoNumber, String tdcChannel, String station, String plane, String slab,
            String pmt, String expectedResponse) {
        // assertEquals("cabling", expectedResponse, m_cabling.setDetector(name,
        // adcComputerId, adcGeoNumber,
        // adcChannel, tdcComputerId, tdcGeoNumber, tdcChannel, station, plane,
        // slab, pmt)); //TODO
    }

    private void setTracker(String name, String vlsbComputerId, String vlsbGeoNumber, String vlsbChannel,
            String trackerNo, String station, String plane, String channel, String expectedResponse) {
        String xml = "<cabling><tracker vlsbComputerId='" + vlsbComputerId + "' vlsbGeoNumber='" + vlsbGeoNumber
                + "' vlsbChannel='" + vlsbChannel + "' trackerNumber='" + trackerNo + "' station='" + station
                + "' plane='" + plane + "' channel='" + channel + "'/></cabling>";
        assertEquals("cabling", expectedResponse, m_cabling.setTracker(name, xml));
    }

    // private void setTracker(String name, String vlsbComputerId, String
    // vlsbGeoNumber, String vlsbChannel,
    // String trackerNo, String station, String plane, String channel, String
    // expectedResponse) {
    // // assertEquals("cabling", expectedResponse, m_cabling.setTracker(name,
    // // vlsbComputerId, vlsbGeoNumber,
    // // vlsbChannel, trackerNo, station, plane, channel)); //TODO
    // }

}
