package mice.cdb.service.cabling;

import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class CablingTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static Cabling m_cabling;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(CablingTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_cabling = new CablingImpl();
        testClassReady();
    }

    @After
    public void beamline_After() {
        endOfTest();
    }

    @AfterClass
    public static void beamline_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_cabling.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getCablingForDate(device, timestamp)
    @Test
    public final void testGetCablingForDateConntrol() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='CONTROL'></cabling>";
        getCablingForDate("Control", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'>"
                + "<module number='1'><channel number='1' name='c11'/><channel number='2' name='c12'/></module>"
                + "<module number='2'><channel number='1' name='c13'/></module></crate>" + "<crate number='2'>"
                + "<module number='3'><channel number='1' name='c14'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForDate("Control", "2010-10-27 16:29:22.792", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCablingForDateDetector() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='DETECTOR'></cabling>";
        getCablingForDate("Ckov A", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCablingForDate("Ckov A", "2010-10-26 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCablingForDateTracker() throws Exception {
        // before first entry
        String expectedResponse = "<cabling type='TRACKER'></cabling>";
        getCablingForDate("Tracker 1", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='110'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='141' station='151' plane='161' channel='171'/></vlsbGeoNumber><vlsbGeoNumber number='121'><vlsbChannel number='130' trackerNumber='142' station='152' plane='162' channel='172'/><vlsbChannel number='131' trackerNumber='143' station='153' plane='163' channel='173'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='111'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='144' station='154' plane='164' channel='174'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2010-10-27 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2010-10-27 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForDate("Tracker 1", "2011-10-27 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCablingForDateError() throws Exception {
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        getCablingForDate("Junk", "20c10-10-25 16:29:22.789", expectedResponse);
        getCablingForDate("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: Ju'nk</error>";
        getCablingForDate("Ju'nk", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCablingForDate("", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCablingForDate("Junk", "2010-10-25 16:29:22.789", expectedResponse);
    }

    private void getCablingForDate(String device, String timestamp, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCablingForDate(device, timestamp));
    }

    // getCablingForRun(device, runNo)
    @Test
    public final void testGetCablingForRunControl() throws Exception {
        String expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c11'/><channel number='2' name='c12'/></module><module number='2'><channel number='1' name='c13'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c14'/></module></crate></cabling>";
        getCablingForRun("control", "2", expectedResponse);
        expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCablingForRun("control", "3", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCablingForRunDetector() throws Exception {
        String expectedResponse = ""; // TODO
        getCablingForRun("Ckov A", "2", expectedResponse);
        expectedResponse = ""; // TODO
        getCablingForRun("Ckov A", "3", expectedResponse);
    }

    @Test
    public final void testGetCablingForRunTracker() throws Exception {
        String expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='110'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='141' station='151' plane='161' channel='171'/></vlsbGeoNumber><vlsbGeoNumber number='121'><vlsbChannel number='130' trackerNumber='142' station='152' plane='162' channel='172'/><vlsbChannel number='131' trackerNumber='143' station='153' plane='163' channel='173'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='111'><vlsbGeoNumber number='120'><vlsbChannel number='130' trackerNumber='144' station='154' plane='164' channel='174'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForRun("Tracker 1", "2", expectedResponse);
        expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCablingForRun("Tracker 1", "3", expectedResponse);
    }

    @Test
    public final void testGetCablingForRunError() throws Exception {
        String expectedResponse = "<error>Run 0 not found.</error>";
        getCablingForRun("Junk", "0", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        getCablingForRun("Junk", "a", expectedResponse);
        expectedResponse = "<error>Value: is not a valid int</error>";
        getCablingForRun("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: J'unk</error>";
        getCablingForRun("J'unk", "1", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCablingForRun("", "1", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCablingForRun("Junk", "1", expectedResponse);
    }

    private void getCablingForRun(String device, String runNo, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCablingForRun(device, runNo));
    }

    // getCurrentCabling(device)
    @Test
    public final void testGetCurrentCablingControl() throws Exception {
        String expectedResponse = "<cabling type='CONTROL'><crate number='1'><module number='1'><channel number='1' name='c21'/><channel number='2' name='c22'/></module><module number='2'><channel number='1' name='c23'/></module></crate><crate number='2'><module number='3'><channel number='1' name='c24'/></module></crate></cabling>";
        getCurrentCabling("control", expectedResponse);
    }

    @Ignore
    @Test
    public final void testGetCurrentCablingDetector() throws Exception {
        String expectedResponse = "Ckov b"; // TODO
        getCurrentCabling("", expectedResponse);
    }

    @Test
    public final void testGetCurrentCablingTracker() throws Exception {
        String expectedResponse = "<cabling type='TRACKER'><vlsbComputerId number='210'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='241' station='251' plane='261' channel='271'/></vlsbGeoNumber><vlsbGeoNumber number='221'><vlsbChannel number='230' trackerNumber='242' station='252' plane='262' channel='272'/><vlsbChannel number='231' trackerNumber='243' station='253' plane='263' channel='273'/></vlsbGeoNumber></vlsbComputerId><vlsbComputerId number='211'><vlsbGeoNumber number='220'><vlsbChannel number='230' trackerNumber='244' station='254' plane='264' channel='274'/></vlsbGeoNumber></vlsbComputerId></cabling>";
        getCurrentCabling("Tracker 1", expectedResponse);
    }

    @Test
    public final void testGetCurrentCablingError() throws Exception {
        String expectedResponse = "<error>Unknown device:Junk</error>";
        getCurrentCabling("Junk", expectedResponse);
    }

    private void getCurrentCabling(String device, String expectedResponse) {
        assertEquals("cabling", expectedResponse, m_cabling.getCurrentCabling(device));
    }

    // listDevices()
    @Test
    public final void testListDevices() throws Exception {
        String result = m_cabling.listDevices();
        String expectedResponse = "<cabling><deviceList><device name='CONTROL'/>"
                + "<device name='CKOV A'/><device name='CKOV B'/><device name='EMR'/><device name='KL'/><device name='SCALERS'/>"
                + "<device name='TOF0'/><device name='TOF1'/><device name='TOF2'/><device name='TRIGGER'/><device name='TRIGGER REQUEST'/>"
                + "<device name='TRACKER 1'/><device name='TRACKER 2'/></deviceList></cabling>";
        assertEquals("cabling", expectedResponse, result);
    }

}
