package mice.cdb.service.calibration;

import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class CalibrationTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static Calibration m_calibration;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(CalibrationTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_calibration = new CalibrationImpl();
        testClassReady();
    }

    @After
    public void beamline_After() {
        endOfTest();
    }

    @AfterClass
    public static void beamline_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_calibration.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getCalibrationForDate(device, timestamp)
    @Ignore
    @Test
    public final void testGetCalibrationForDateDetector() throws Exception {
        // before first entry
        String expectedResponse = "<calibration type='DETECTOR'></calibration>";
        getCalibrationForDate("Ckov A", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = ""; // TODO
        getCalibrationForDate("Ckov A", "2010-10-26 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCalibrationForDate("Ckov A", "2010-10-26 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = ""; // TODO
        getCalibrationForDate("Ckov A", "2010-10-26 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCalibrationForDateTracker() throws Exception {
        // before first entry
        String expectedResponse = "<calibration type='TRACKER'></calibration>";
        getCalibrationForDate("Tracker 1", "2010-10-25 16:29:22.789", expectedResponse);

        // first entry
        expectedResponse = "<calibration type='TRACKER'><board number='110'><bank number='120'><channel number='130' adcPedestal='141' adcGain='151' tdcPedestal='161' tdcSlope='171'/></bank><bank number='121'><channel number='130' adcPedestal='142' adcGain='152' tdcPedestal='162' tdcSlope='172'/><channel number='131' adcPedestal='143' adcGain='153' tdcPedestal='163' tdcSlope='173'/></bank></board><board number='111'><bank number='120'><channel number='130' adcPedestal='144' adcGain='154' tdcPedestal='164' tdcSlope='174'/></bank></board></calibration>";
        getCalibrationForDate("Tracker 1", "2010-10-27 16:29:22.790", expectedResponse);

        // second entry
        expectedResponse = "<calibration type='TRACKER'><board number='210'><bank number='220'><channel number='230' adcPedestal='241' adcGain='251' tdcPedestal='261' tdcSlope='271'/></bank><bank number='221'><channel number='230' adcPedestal='242' adcGain='252' tdcPedestal='262' tdcSlope='272'/><channel number='231' adcPedestal='243' adcGain='253' tdcPedestal='263' tdcSlope='273'/></bank></board><board number='211'><bank number='220'><channel number='230' adcPedestal='244' adcGain='254' tdcPedestal='264' tdcSlope='274'/></bank></board></calibration>";
        getCalibrationForDate("Tracker 1", "2010-10-27 16:29:22.791", expectedResponse);

        // second entry
        expectedResponse = "<calibration type='TRACKER'><board number='210'><bank number='220'><channel number='230' adcPedestal='241' adcGain='251' tdcPedestal='261' tdcSlope='271'/></bank><bank number='221'><channel number='230' adcPedestal='242' adcGain='252' tdcPedestal='262' tdcSlope='272'/><channel number='231' adcPedestal='243' adcGain='253' tdcPedestal='263' tdcSlope='273'/></bank></board><board number='211'><bank number='220'><channel number='230' adcPedestal='244' adcGain='254' tdcPedestal='264' tdcSlope='274'/></bank></board></calibration>";
        getCalibrationForDate("Tracker 1", "2011-10-27 16:29:22.792", expectedResponse);
    }

    @Test
    public final void testGetCalibrationForDateError() throws Exception {
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        getCalibrationForDate("Junk", "20c10-10-25 16:29:22.789", expectedResponse);
        getCalibrationForDate("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: Ju'nk</error>";
        getCalibrationForDate("Ju'nk", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCalibrationForDate("", "2010-10-25 16:29:22.789", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCalibrationForDate("Junk", "2010-10-25 16:29:22.789", expectedResponse);
    }

    private void getCalibrationForDate(String device, String timestamp, String expectedResponse) {
        assertEquals("calibration", expectedResponse, m_calibration.getCalibrationForDate(device, timestamp));
    }

    // getCalibrationForRun(device, runNo)
    @Ignore
    @Test
    public final void testGetCalibrationForRunDetector() throws Exception {
        String expectedResponse = "<calibration type='DETECTOR'></calibration>"; // TODO
        getCalibrationForRun("Ckov A", "2", expectedResponse);
        expectedResponse = ""; // TODO
        getCalibrationForRun("Ckov A", "3", expectedResponse);
    }

    @Test
    public final void testGetCalibrationForRunTracker() throws Exception {
        String expectedResponse = "<calibration type='TRACKER'><board number='110'><bank number='120'><channel number='130' adcPedestal='141' adcGain='151' tdcPedestal='161' tdcSlope='171'/></bank><bank number='121'><channel number='130' adcPedestal='142' adcGain='152' tdcPedestal='162' tdcSlope='172'/><channel number='131' adcPedestal='143' adcGain='153' tdcPedestal='163' tdcSlope='173'/></bank></board><board number='111'><bank number='120'><channel number='130' adcPedestal='144' adcGain='154' tdcPedestal='164' tdcSlope='174'/></bank></board></calibration>";
        getCalibrationForRun("Tracker 1", "2", expectedResponse);
        expectedResponse = "<calibration type='TRACKER'><board number='210'><bank number='220'><channel number='230' adcPedestal='241' adcGain='251' tdcPedestal='261' tdcSlope='271'/></bank><bank number='221'><channel number='230' adcPedestal='242' adcGain='252' tdcPedestal='262' tdcSlope='272'/><channel number='231' adcPedestal='243' adcGain='253' tdcPedestal='263' tdcSlope='273'/></bank></board><board number='211'><bank number='220'><channel number='230' adcPedestal='244' adcGain='254' tdcPedestal='264' tdcSlope='274'/></bank></board></calibration>";
        getCalibrationForRun("Tracker 1", "3", expectedResponse);
    }

    @Test
    public final void testGetCalibrationForRunError() throws Exception {
        String expectedResponse = "<error>Run 0 not found.</error>";
        getCalibrationForRun("Junk", "0", expectedResponse);
        expectedResponse = "<error>Value:a is not a valid int</error>";
        getCalibrationForRun("Junk", "a", expectedResponse);
        expectedResponse = "<error>Value: is not a valid int</error>";
        getCalibrationForRun("Junk", "", expectedResponse);
        expectedResponse = "<error>String contained non-aphanumeric/non-white space characters: J'unk</error>";
        getCalibrationForRun("J'unk", "1", expectedResponse);
        expectedResponse = "<error>String cannot be null or empty</error>";
        getCalibrationForRun("", "1", expectedResponse);
        expectedResponse = "<error>Unknown device:Junk</error>";
        getCalibrationForRun("Junk", "1", expectedResponse);
    }

    private void getCalibrationForRun(String device, String runNo, String expectedResponse) {
        assertEquals("calibration", expectedResponse, m_calibration.getCalibrationForRun(device, runNo));
    }

    // getCurrentCalibration(device)
    @Ignore
    @Test
    public final void testGetCurrentCalibrationDetector() throws Exception {
        String expectedResponse = ""; // TODO
        getCurrentCabling("Ckov b", expectedResponse);
    }

    @Test
    public final void testGetCurrentCalibrationTracker() throws Exception {
        String expectedResponse = "<calibration type='TRACKER'><board number='210'><bank number='220'><channel number='230' adcPedestal='241' adcGain='251' tdcPedestal='261' tdcSlope='271'/></bank><bank number='221'><channel number='230' adcPedestal='242' adcGain='252' tdcPedestal='262' tdcSlope='272'/><channel number='231' adcPedestal='243' adcGain='253' tdcPedestal='263' tdcSlope='273'/></bank></board><board number='211'><bank number='220'><channel number='230' adcPedestal='244' adcGain='254' tdcPedestal='264' tdcSlope='274'/></bank></board></calibration>";
        getCurrentCabling("Tracker 1", expectedResponse);
    }

    @Test
    public final void testGetCurrentCalibrationJunk() throws Exception {
        String expectedResponse = "<error>Unknown device:Junk</error>";
        getCurrentCabling("Junk", expectedResponse);
    }

    private void getCurrentCabling(String device, String expectedResponse) {
        assertEquals("calibration", expectedResponse, m_calibration.getCurrentCalibration(device));
    }

    // listDevices()
    @Test
    public final void testListDevices() throws Exception {
        String result = m_calibration.listDevices();
        String expectedResponse = "<calibration><deviceList><device name='CKOV A'/><device name='CKOV B'/><device name='EMR'/><device name='KL'/><device name='SCALERS'/><device name='TOF0'/><device name='TOF1'/><device name='TOF2'/><device name='TRIGGER'/><device name='TRIGGER REQUEST'/><device name='TRACKER 1'/><device name='TRACKER 2'/></deviceList></calibration>";
        assertEquals("calibration", expectedResponse, result);
    }

}
