package mice.cdb.service.control;

import static org.junit.Assert.assertEquals;

import java.sql.Timestamp;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.RTest;
import utils.TestSetup;

public class ControlEmptySMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static ControlSuperMouse m_control;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(ControlEmptySMTest.class);
        TestSetup.initializeServer();
        m_control = new ControlSuperMouseImpl();
        testClassReady();
    }

    // @Before
    // public void control_Before() throws Exception {
    // TestSetup.initializeServer();
    // }

    @After
    public void control_After() {
        endOfTest();
    }

    @AfterClass
    public static void control_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_control.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getControls()
    @Test
    public final void testGetControls() throws Exception {
        String result = m_control.getControls();
        assertEquals("Controls", "<controls></controls>", result);
    }

    // getControlsForCrate()
    @Test
    public final void testGetControlsForCrate() throws Exception {
        String result = m_control.getControlsForCrate("1");
        String expectedResponse = "<controls></controls>";
        assertEquals("Controls", expectedResponse, result);
    }

    // getPreviousSettings(timestamp)
    @Test
    public final void testGetPreviousSettings() throws Exception {
        String timestamp = new Timestamp(System.currentTimeMillis()).toString();
        String result = m_control.getPreviousSettings(timestamp);
        String expectedResponse = "<controls></controls>";
        assertEquals("Expecting empty message", expectedResponse, result);
    }

    // setParameter(crate, module, channel, name, value)
    @Test
    public final void testSetParameter() throws Exception {
        String result = m_control.setParameter("9", "1", "1", "p1", "111");
        assertEquals("Controls", "<ok>Parameter set</ok>", result);
        result = m_control.setParameter("9", "1", "1", "p1", "1");
        assertEquals("Controls", "<error>Entry for 9, 1, 1, p1 already exists</error>", result);
    }

    // updateParameter(crate, module, channel, name, value)
    @Test
    public final void testUpdateParameter() throws Exception {
        String result = m_control.updateParameter("10", "1", "1", "none", "1");
        assertEquals("Controls", "<error>Entry for 10, 1, 1, none does not exist</error>", result);
    }

}
