package mice.cdb.service.control;

import static org.junit.Assert.assertEquals;

import java.sql.Timestamp;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class ControlSMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static ControlSuperMouse m_control;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(ControlSMTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_control = new ControlSuperMouseImpl();
        testClassReady();
    }

    @After
    public void control_After() {
        endOfTest();
    }

    @AfterClass
    public static void control_AfterClass() {
        endOfTestClass();
    }

    // GetStatus()
    @Test
    public final void testGetStatus() throws Exception {
        String result = m_control.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    // getControls()
    @Test
    public final void testGetControls() throws Exception {
        String result = m_control.getControls();
        String expectedResponse = "<controls><crate name='1'><channel name='1' module='1' remoteChannelName='c21'><parameter name='param1' value='111'/><parameter name='param2' value='112'/><parameter name='param3' value='113'/><parameter name='param4' value='114'/><parameter name='param5' value='115'/></channel><channel name='2' module='1' remoteChannelName='c22'><parameter name='param1' value='121'/><parameter name='param2' value='122'/><parameter name='param3' value='123'/><parameter name='param4' value='124'/><parameter name='param5' value='125'/></channel><channel name='1' module='2' remoteChannelName='c23'><parameter name='param1' value='211'/><parameter name='param2' value='212'/><parameter name='param3' value='213'/><parameter name='param4' value='214'/><parameter name='param5' value='215'/></channel></crate><crate name='2'><channel name='1' module='3' remoteChannelName='c24'><parameter name='param1' value='221'/><parameter name='param2' value='222'/><parameter name='param3' value='223'/><parameter name='param4' value='224'/><parameter name='param5' value='225'/></channel></crate></controls>";
        assertEquals("Controls", expectedResponse, result);
    }

    // getControlsForCrate()
    @Test
    public final void testGetControlsForCrate() throws Exception {
        String result = m_control.getControlsForCrate("2");
        String expectedResponse = "<controls><crate name='2'><channel name='1' module='3' remoteChannelName='c24'><parameter name='param1' value='221'/><parameter name='param2' value='222'/><parameter name='param3' value='223'/><parameter name='param4' value='224'/><parameter name='param5' value='225'/></channel></crate></controls>";
        assertEquals("Controls", expectedResponse, result);
    }

    @Test
    public final void testGetControlsForCrateBadData() throws Exception {
        String result = m_control.getControlsForCrate("9999");
        String expectedResponse = "<controls></controls>";
        assertEquals("Controls", expectedResponse, result);
        result = m_control.getControlsForCrate("A");
        expectedResponse = "<error>Value:A is not a valid int</error>";
        assertEquals("Controls", expectedResponse, result);
    }

    // getPreviousSettings(timestamp)
    @Test
    public final void testGetPreviousSettings() throws Exception {
        String timestamp = new Timestamp(System.currentTimeMillis()).toString();
        String result = m_control.getPreviousSettings(timestamp);
        String expectedResponse = "<controls><crate name='1'><channel name='1' module='1' remoteChannelName='c21'><parameter name='param1' value='111'/><parameter name='param2' value='112'/><parameter name='param3' value='113'/><parameter name='param4' value='114'/><parameter name='param5' value='115'/></channel><channel name='2' module='1' remoteChannelName='c22'><parameter name='param1' value='121'/><parameter name='param2' value='122'/><parameter name='param3' value='123'/><parameter name='param4' value='124'/><parameter name='param5' value='125'/></channel><channel name='1' module='2' remoteChannelName='c23'><parameter name='param1' value='211'/><parameter name='param2' value='212'/><parameter name='param3' value='213'/><parameter name='param4' value='214'/><parameter name='param5' value='215'/></channel></crate><crate name='2'><channel name='1' module='3' remoteChannelName='c24'><parameter name='param1' value='221'/><parameter name='param2' value='222'/><parameter name='param3' value='223'/><parameter name='param4' value='224'/><parameter name='param5' value='225'/></channel></crate></controls>"; // TODO
        assertEquals("Controls", expectedResponse, result);
    }

    @Test
    public final void testGetPreviousSettingsBadData() throws Exception {
        getBadPreviousSettings("20.10-10-25 12:03:03.187");
        getBadPreviousSettings("2010-10-25 12:03:03.18700000Z");
        getBadPreviousSettings("20A10-10-25 12:03:03.18700000Z");
        getBadPreviousSettings("20A10-10-25 12:03:03.187'0'00Z");
    }

    private void getBadPreviousSettings(String timestamp) throws Exception {
        String result = m_control.getPreviousSettings(timestamp);
        String message = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", message, result);
    }

    // setParameter(rack, crate, module, channel, name, value)
    @Test
    public final void testSetParameter() throws Exception {
        String result = m_control.setParameter("9", "1", "1", "p1", "111");
        assertEquals("Controls", "<ok>Parameter set</ok>", result);
        result = m_control.setParameter("9", "1", "1", "p1", "1");
        assertEquals("Controls", "<error>Entry for 9, 1, 1, p1 already exists</error>", result);
    }

    // updateParameter(rack, crate, module, channel, name, value)
    @Test
    public final void testUpdateParameter() throws Exception {
        String result = m_control.updateParameter("10", "1", "1", "none", "1");
        assertEquals("Controls", "<error>Entry for 10, 1, 1, none does not exist</error>", result);
        result = m_control.updateParameter("1", "1", "1", "param1", "111");
        assertEquals("Controls", "<ok>Parameter updated</ok>", result);
    }

}
