package mice.cdb.service.geometry;

import static org.junit.Assert.assertEquals;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.LoadData;
import utils.RTest;
import utils.TestSetup;

public class GeometrySMTest extends RTest {

    mice.cdb.service.ServerConfig m_serverConfig;

    private static GeometrySuperMouse m_geometry;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(GeometrySMTest.class);
        TestSetup.initializeServer();
        LoadData.insert();
        m_geometry = new GeometrySuperMouseImpl();
        testClassReady();
    }

    @After
    public void a1_After() {
        endOfTest();
    }

    @AfterClass
    public static void a1_AfterClass() {
        endOfTestClass();
    }

    @Test
    public final void testGetStatus() throws Exception {
        String result = m_geometry.getStatus();
        assertEquals("Status", "<ok>OK</ok>", result);
    }

    @Test
    public final void testGetIds() throws Exception {
        String result = m_geometry.getIds("2010-10-25 16:29:22.791", null);
        assertEquals(
                "Ids",
                "<ids><id name='1' validFrom='2010-10-25 16:29:22.791' created='2010-10-25 16:29:22.791' notes='blar 1' technicalDrawingName='null'/><id name='4' validFrom='2010-10-25 16:29:22.791' created='2010-10-27 16:29:22.791' notes='blar 4' technicalDrawingName='null'/><id name='6' validFrom='2010-10-25 16:29:22.791' created='2010-10-29 16:29:22.791' notes='blar 6' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-25 16:29:22.792", null);
        assertEquals(
                "Ids",
                "<ids><id name='1' validFrom='2010-10-25 16:29:22.791' created='2010-10-25 16:29:22.791' notes='blar 1' technicalDrawingName='null'/><id name='4' validFrom='2010-10-25 16:29:22.791' created='2010-10-27 16:29:22.791' notes='blar 4' technicalDrawingName='null'/><id name='6' validFrom='2010-10-25 16:29:22.791' created='2010-10-29 16:29:22.791' notes='blar 6' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-26 16:29:22.790", null);
        assertEquals(
                "Ids",
                "<ids><id name='1' validFrom='2010-10-25 16:29:22.791' created='2010-10-25 16:29:22.791' notes='blar 1' technicalDrawingName='null'/><id name='4' validFrom='2010-10-25 16:29:22.791' created='2010-10-27 16:29:22.791' notes='blar 4' technicalDrawingName='null'/><id name='6' validFrom='2010-10-25 16:29:22.791' created='2010-10-29 16:29:22.791' notes='blar 6' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-26 16:29:22.791", null);
        assertEquals(
                "Ids",
                "<ids><id name='5' validFrom='2010-10-26 16:29:22.791' created='2010-10-28 16:29:22.791' notes='blar 5' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-26 16:29:22.792", null);
        assertEquals(
                "Ids",
                "<ids><id name='5' validFrom='2010-10-26 16:29:22.791' created='2010-10-28 16:29:22.791' notes='blar 5' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-25 16:29:22.792", "2010-10-26 16:29:22.792");
        assertEquals(
                "Ids",
                "<ids><id name='1' validFrom='2010-10-25 16:29:22.791' created='2010-10-25 16:29:22.791' notes='blar 1' technicalDrawingName='null'/><id name='4' validFrom='2010-10-25 16:29:22.791' created='2010-10-27 16:29:22.791' notes='blar 4' technicalDrawingName='null'/><id name='6' validFrom='2010-10-25 16:29:22.791' created='2010-10-29 16:29:22.791' notes='blar 6' technicalDrawingName='null'/><id name='5' validFrom='2010-10-26 16:29:22.791' created='2010-10-28 16:29:22.791' notes='blar 5' technicalDrawingName='null'/></ids>",
                result);
        result = m_geometry.getIds("2010-10-27 16:29:22.792", "2010-10-28 16:29:22.792");
        assertEquals(
                "Ids",
                "<ids><id name='2' validFrom='2010-10-27 16:29:22.791' created='2010-10-27 16:29:20.791' notes='blar 2' technicalDrawingName='null'/><id name='3' validFrom='2010-10-27 16:29:22.791' created='2010-10-27 16:29:21.791' notes='blar 3' technicalDrawingName='null'/></ids>",
                result);
    }

    @Test
    public final void testGetIdsTooEarly() throws Exception {
        String result = m_geometry.getIds("2010-10-25 16:29:22.790", null);
        assertEquals("Ids", "<ids></ids>", result);
        result = m_geometry.getIds("2010-10-25 16:29:22.789", "2010-10-25 16:29:22.790");
        assertEquals("Ids", "<ids></ids>", result);
        result = m_geometry.getIds("2000-01-01 16:29:22.790", "2010-10-25 16:29:22.791");
        assertEquals(
                "Ids",
                "<ids><id name='1' validFrom='2010-10-25 16:29:22.791' created='2010-10-25 16:29:22.791' notes='blar 1' technicalDrawingName='null'/><id name='4' validFrom='2010-10-25 16:29:22.791' created='2010-10-27 16:29:22.791' notes='blar 4' technicalDrawingName='null'/><id name='6' validFrom='2010-10-25 16:29:22.791' created='2010-10-29 16:29:22.791' notes='blar 6' technicalDrawingName='null'/></ids>",
                result);
    }

    @Test
    public final void testGetIdsBadData() throws Exception {
        getBadIds("20.10-10-25 12:03:03.187", null);
        getBadIds("2010-10-25 12:03:03.18700000Z", null);
        getBadIds("20A10-10-25 12:03:03.18700000Z", null);
        getBadIds("20A10-10-25 12:03:03.187'0'00Z", null);
        getBadIds(null, null);
        getBadIds("2000-10-27 16:29:22.791", "20.10-10-25 12:03:03.187");
        getBadIds("2000-10-27 16:29:22.791", "2010-10-25 12:03:03.18700000Z");
        getBadIds("2000-10-27 16:29:22.791", "20A10-10-25 12:03:03.18700000Z");
        getBadIds("2000-10-27 16:29:22.791", "20A10-10-25 12:03:03.187'0'00Z");
    }

    @Test
    public final void testGetIdsBadStoptime() throws Exception {
        String result = m_geometry.getIds("2010-10-27 16:29:22.791", "2010-10-27 16:29:22.790");
        assertEquals("Ids", "<error>Start time must be before stop time.</error>", result);
    }

    @Test
    public final void testCurrentGDML() throws Exception {
        String result = new String(m_geometry.getCurrentGDML());
        assertEquals("GDML", "gdml - 3", result);
    }

    @Test
    public final void testgetGDMLForId() throws Exception {
        String id = "5";
        String result = new String(m_geometry.getGDMLForId(id));
        assertEquals("GDML", "gdml - " + id, result);
    }

    @Test
    public final void testgetGDMLForIdBadData() throws Exception {
        String id = "i";
        String result = new String(m_geometry.getGDMLForId(id));
        assertEquals("GDML", "<error>Value:" + id + " is not a valid int</error>", result);
    }

    @Test
    public final void testGDMLForRun() throws Exception {
        String runNo = "2";
        String result = new String(m_geometry.getGDMLForRun(runNo));
        assertEquals("GDML", "gdml - 6", result);
    }

    @Test
    public final void testGDMLForRunBadData() throws Exception {
        String runNo = "i";
        String result = new String(m_geometry.getGDMLForRun(runNo));
        assertEquals("GDML", "<error>Value:" + runNo + " is not a valid int</error>", result);
    }

    @Test
    public final void testSetGDML() throws Exception {
        byte[] gdml = "my test GDML".getBytes();
        String result = m_geometry.setGDML(gdml, "2011-01-01 16:29:22.791", "junk", "TD");
        assertEquals("GDML", "<ok>Inserted GDML, 12 bytes</ok>", result);
    }

    @Test
    public final void testSetGDMLBadData() throws Exception {
        byte[] gdml = "my test GDML".getBytes();
        String result = m_geometry.setGDML(gdml, null, null, null);
        assertEquals("GDML", "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>", result);
    }

    private void getBadIds(String startTime, String stopTime) throws Exception {
        String result = m_geometry.getIds(startTime, stopTime);
        String expectedResponse = "<error>Timestamp format must be yyyy-mm-dd hh:mm:ss[.fffffffff]</error>";
        assertEquals("Expecting error message", expectedResponse, result);
    }

}
