package utils;

import static org.junit.Assert.fail;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryPoolMXBean;
import java.util.List;
import java.util.regex.Pattern;

import mice.cdb.service.CDBException;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;

import org.apache.log4j.Logger;

public abstract class RTest {

    private static final Logger TEST_LOG = Logger.getLogger(TestSetup.TEST_LOGGER);
    private static long s_groupStart;
    private static String s_groupName;
    private static int s_num;
    private static long s_oneTestStart;
    private static States state = States.OUTSIDE;

    private enum States {
        OUTSIDE, INSETUP, INTEST
    }

    protected void checkPattern(String pattern, Exception e) throws Exception {
        if (!Pattern.matches(pattern, e.getMessage())) {
            fail("Bad Match\n" + e.getMessage() + "\n" + pattern);
        }
    }

    protected static void startOfTestClass(Class<?> aclass) {
        if (state != States.OUTSIDE) {
            TEST_LOG.error(state + " is inappropriate state for startOfTestClass() call - was set by " + s_groupName);
        }
        state = States.INSETUP;
        s_groupName = aclass.getName();

        TEST_LOG.info("Test " + s_groupName + " group starting");
        s_groupStart = System.currentTimeMillis();
        s_num = 0;
    }

    protected static void testClassReady() {
        checkState("testClassReady", States.INSETUP, States.INTEST);
        TEST_LOG.info("Test " + s_groupName + " group ready: " + (System.currentTimeMillis() - s_groupStart) / 1000.
                + " seconds.");
        TEST_LOG.info("Test " + s_groupName + " threads currently running: " + Thread.getAllStackTraces().size());
        s_oneTestStart = System.currentTimeMillis();
    }

    protected void endOfTest() {
        checkState("endOfTest", States.INTEST, States.INTEST);
        TEST_LOG.info("Test " + s_groupName + " #" + ++s_num + " finished "
                + (System.currentTimeMillis() - s_oneTestStart) / 1000. + " seconds.");
        s_oneTestStart = System.currentTimeMillis(); // Remember as start of
                                                     // next test
    }

    protected static void endOfTestClass() {
        checkState("endOfTestClass", States.INTEST, States.OUTSIDE);
        TEST_LOG.info("Test " + s_groupName + " group total time: " + (System.currentTimeMillis() - s_groupStart)
                / 1000. + " seconds.\n");
    }

    private static void checkState(String op, States before, States after) {
        if (state != before) {
            TEST_LOG.error(state + " is inappropriate state for " + op + " call. It should be " + before
                    + ". It was set by " + s_groupName);
            TEST_LOG.error("Results will not be accurate for this test group");
        }
        state = after;
    }

    @SuppressWarnings("unused")
    private void printShortMemStat() throws CDBException {
        ServerConfig m_serverConfig;
        m_serverConfig = ServerConfig.getInstance();
        String poolToWatch = m_serverConfig.getString(ServerConstants.SERVER_POOL_TO_WATCH);

        List<MemoryPoolMXBean> mpbeans = ManagementFactory.getMemoryPoolMXBeans();
        for (MemoryPoolMXBean mpbean : mpbeans) {
            if (mpbean.getName().equals(poolToWatch)) {
                System.out.print("Pool Name: " + mpbean.getName());
                System.out.print(",    Type: " + mpbean.getType());
                System.out.println(",    Valid: " + mpbean.isValid());
                System.out.print("  Usage:           " + mpbean.getUsage().getUsed());
                System.out.print("  Peak Usage:      " + mpbean.getPeakUsage().getUsed());
                if (mpbean.getCollectionUsage() != null) {
                    System.out.print("  CollectionUsage: " + mpbean.getCollectionUsage().getUsed());
                }
                if (mpbean.isUsageThresholdSupported()) {
                    if (mpbean.isUsageThresholdExceeded()) {
                        System.out.print(" U");
                    }
                }
                if (mpbean.isCollectionUsageThresholdSupported()) {
                    if (mpbean.isCollectionUsageThresholdExceeded()) {
                        System.out.print("  C");
                    }
                }
                System.out.println();
            }
        }
    }

}
